"""Test functions for plot command."""

from easyviztest import *


class test_plot_basic(EasyvizTestCase):
    def check_plot_default(self):
        x,y,v,w = self.get_line_data()
        plot(x,y)
        title("plot(x,y)")
        n()

    def check_plot_only_y(self):
        x,y,v,w = self.get_line_data()
        plot(y)
        title("plot(y)")
        n()
        
    def check_plot_kwarg_x(self):
        x,y,v,w = self.get_line_data()
        plot(y,x=x)
        title("plot(y,x=x)")
        n()
        
    def check_plot_kwarg_auto_x(self):
        x,y,v,w = self.get_line_data()
        plot(y,x='auto')
        title("plot(y,x='auto')")
        n()
        
    def check_plot_two_lines(self):
        x,y,v,w = self.get_line_data()
        plot(x,y,x,v)
        title("plot(x,y,x,v)")
        n()
        
    def check_plot_two_lines_kwarg_x(self):
        x,y,v,w = self.get_line_data()
        plot(y,v,x=x)
        title("plot(y,v,x=x)")
        n()
        
    def check_plot_two_lines_kwarg_auto_x(self):
        x,y,v,w = self.get_line_data()
        plot(y,v,x='auto')
        title("plot(y,v,x='auto')")
        n()

    def check_plot_multiple_lines(self):
        x,y,v,w = self.get_line_data()
        plot(x,y,x,v,x,w,x,x,x,-x)
        title("plot(x,y,x,v,x,w,x,x,x,-x)")
        n()
        
    def check_plot_multiple_lines_kwarg_x(self):
        x,y,v,w = self.get_line_data()
        plot(y,v,w,x,-x,x=x)
        title("plot(y,v,w,x,-x,x=x)")
        n()
        
    def check_plot_multiple_lines_kwarg_auto_x(self):
        x,y,v,w = self.get_line_data()
        plot(y,v,w,x,-x,x='auto')
        title("plot(y,v,w,x,-x,x='auto')")
        n()


class test_plot_format_string(EasyvizTestCase):
    def check_plot_multiple_lines_fmt(self):
        x,y,v,w = self.get_line_data()
        plot(x,y,'r-',x,v,'b--',x,w,'g--',x,x,'x',x,-x,'k:')
        title("plot(x,y,'r-',x,v,'b--',x,w,'g--',x,x,'x',x,-x,'k:')")
        n()

    def check_plot_multiple_lines_fmt_kwarg_x(self):
        x,y,v,w = self.get_line_data()
        plot(y,'r-',v,'b--',w,'g--',x,'x',-x,'k:',x=x)
        title("plot(y,'r-',v,'b--',w,'g--',x,'x',-x,'k:',x=x)")
        n()

    def check_plot_multiple_lines_fmt_kwarg_auto_x(self):
        x,y,v,w = self.get_line_data()
        plot(y,'r-',v,'b--',w,'g--',x,'x',-x,'k:',x='auto')
        title("plot(y,'r-',v,'b--',w,'g--',x,'x',-x,'k:',x='auto')")
        n()

    def check_plot_linecolors(self):
        x = linspace(0,1,13)
        setp(show=False)
        colors = Line._colors
        for i,color in enumerate(colors):
            plot(x,x+i,color,hold='on')
        title("different line colors")
        hold('off')
        setp(show=screenplot)
        show()
        n()

    def check_plot_linestyles(self):
        x = linspace(0,1,13)
        setp(show=False)
        styles = Line._linestyles
        for i,style in enumerate(styles):
            plot(x,x+i,style,hold='on')
        title("different line styles")
        hold('off')
        setp(show=screenplot)
        show()
        n()

    def check_plot_linemarkers(self):
        x = linspace(0,1,13)
        setp(show=False)
        markers = Line._markers
        for i,marker in enumerate(markers):
            plot(x,x+i,marker,hold='on')
        title("different line markers")
        hold('off')
        setp(show=screenplot)
        show()
        n()

    def check_plot_linewidth(self):
        x = linspace(0,1,13)
        setp(show=False)
        for i in range(1,10):
            plot(x,x+i,str(i),hold='on')
        title("different line widths")
        hold('off')
        setp(show=screenplot)
        show()
        n()


class test_plot_with_NaNs(EasyvizTestCase):
    def check_plot_NaN_in_list(self):
        plot([0,1,NaN,3,4])
        title("plot([0,1,NaN,3,4])")
        n()
        
    def check_plot_NaN_in_array(self):
        plot(array([0,1,NaN,3,4],float))
        title("plot(array([0,1,NaN,3,4],float))")
        n()


def test_suite(level=1):
    suites = []
    if level > 0:
        suites.append(unittest.makeSuite(test_plot_basic,'check_'))
        suites.append(unittest.makeSuite(test_plot_format_string,'check_'))
        suites.append(unittest.makeSuite(test_plot_with_NaNs,'check_'))
    total_suite = unittest.TestSuite(suites)
    return total_suite

def test(level=10):
    all_tests = test_suite()
    runner = unittest.TextTestRunner()
    runner.run(all_tests)
    return runner

if __name__ == "__main__":
    test()
    raw_input("press enter to exit")



#!/usr/bin/env python

from scitools.std import *

setp(interactive=False)

t = linspace(0,10*pi,201)
plot3(sin(t),cos(t),t,title='A Helix')
show()

figure()
t = linspace(-5,5,501)
x = (2+t**2)*sin(10*t)
y = (2+t**2)*cos(10*t)
z = t
plot3(x,y,z)
grid('on')
xlabel('x(t)')
ylabel('y(t)')
zlabel('z(t)')
title('plot3 example')
show()

figure()
t = linspace(0,15*pi,301)
x = exp(-t/10)*cos(t)
y = exp(-t/10)*sin(t)
z = t
subplot(221);  plot3(x,y,z)
subplot(222);  plot3(x,y,z,view=2)
subplot(223);  plot3(x,y,z,view=[90,90])
subplot(224);  plot3(x,y,z,view=[90,0])
show()

raw_input('Press Return key to quit: ')



#!/usr/bin/env python

"""Demonstration on how to use the surf command."""

from scitools.std import *

xv, yv = ndgrid(linspace(-2,2,21),linspace(-1,1,11))
values = xv**2*yv-2*yv
setp(show=False)
subplot(221);  surf(values)
subplot(222);  surf(xv,yv,values,shading='flat')
subplot(223);  surf(values,shading='interp')
subplot(224);  surfc(values,clevels=10)  # add contours beneath the surface
setp(show=True)
show()

#hardcopy('surf1a.eps')
#hardcopy('surf1a.png')

figure()
# Create a surface plot of a sphere:
n = 32
theta = pi*linspace(-n,n,n)/n
theta = reshape(theta, (n,1))
phi = (pi/2)*linspace(-n,n,n)/n
phi = reshape(phi, (1,n))
x = cos(phi)*cos(theta)
y = cos(phi)*sin(theta)
z = sin(phi)*ones(shape(theta))
surf(x,y,z,axis='equal')

#hardcopy('surf1b.eps')
#hardcopy('surf1b.png')

figure()
xv, yv = ndgrid(linspace(-2,2,41),linspace(-2,2,41))
values = xv*exp(-xv**2-yv**2)
dx, dy = gradient(values)
c = dx + dy
surf(xv,yv,values,c,colorbar='on',axis=[-2,2,-2,2,-0.5,0.5])

#hardcopy('surf1c.eps')
#hardcopy('surf1c.png')

figure()
xv, yv = ndgrid(seq(-2.5,2.5,0.15),seq(-5,5,0.15))
values = 70*yv**2*exp(-xv**2-0.2*yv**2)
surf(xv,yv,values)

#hardcopy('surf1d.eps')
#hardcopy('surf1d.png')

raw_input('Press Return key to quit: ')



#!/usr/bin/python
# part of Lumpy (used by Lumpy.py)

"""Wrapper classes for use with tkinter.

This module provides the following classes:

Gui: a sublass of Tk that provides wrappers for most of the
widget-creating methods from Tk.  The advantage of these wrappers is
that they use Python's optional argument capability to provide
appropriate default values, and that they combine widget creation and
packing into a single step.  They also eliminate the need to name the
parent widget explicitly by keeping track of a current frame and
packing new objects into it.

GuiCanvas: a subclass of Canvas that provides wrappers for most of the
item-creating methods from Canvas.  The advantage of the wrappers
is, again, that they use optional arguments to provide appropriate
defaults, and that they perform coordinate transformations.

Transform: an abstract class that provides basic methods inherited
by CanvasTransform and the other transforms.

CanvasTransform: a transformation that maps standard Cartesian
coordinates onto the 'graphics' coordinates used by Canvas objects.

Callable: the standard recipe from Python Cookbook for encapsulating
a function and its arguments in an object that can be used as a
callback.

The most important idea in this module is using a stack of frames to
avoid keeping track of parent widgets explicitly.


    WIDGET WRAPPERS:

    The Gui class contains wrappers for the widgets in tkinter.
    All of the wrappers invoke widget() to create and pack the new widget.

    The first four positional
    arguments determine how the widget is packed.  Some widgets
    take additional positional arguments.  In most cases, the
    keyword arguments are passed as options to the widget
    constructor.

    The default pack arguments are
    side=TOP, fill=NONE, expand=0, anchor=CENTER

    Widgets that use these defaults can just pass along
    args and options unmolested.  Widgets (like fr and en)
    that want different defaults have to roll the arguments
    in with the other options and then underride them
    (underride means set only if not already set).

    ITEM WRAPPERS:

    GuiCanvas provides wrappers for the canvas item methods.

"""

"""
  Copyright 2005 Allen B. Downey

    This file contains wrapper classes I use with tkinter.  It is
    mostly for my own use; I don't support it, and it is not very
    well documented.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, see
    http://www.gnu.org/licenses/gpl.html or write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
    02110-1301 USA 
"""



from math import *
from Tkinter import *
from tkFont import *

class Gui(Tk):
    """Gui provides wrappers for many of the methods in the Tk
    class; also, it keeps track of the current frame so that
    you can create new widgets without naming the parent frame
    explicitly."""

    def __init__(self, debug=False):
        """initialize the gui.
        turning on debugging changes the behavior of Gui.fr so
        that the nested frame structure is apparent.
        """
        Tk.__init__(self)
        self.debug = debug
        self.frames = [self]           # the stack of nested frames

    # accessing frame as an attribute returns the last frame on
    # the stack
    frame = property(lambda self: self.frames[-1])

    def endfr(self):
        """end the current frame (and return it)"""
        return self.frames.pop()

    # popfr and endgr are synonyms for endfr
    popfr = endfr
    endgr = endfr

    def pushfr(self, frame):
        """push a frame onto the frame stack"""
        self.frames.append(frame)

    def tl(self, **options):
        """make a return a top level window."""
        return Toplevel(**options)

    argnames = ['side', 'fill', 'expand', 'anchor']

    def fr(self, *args, **options):
        """make a return a frame.
        As a side effect, the new frame becomes the current frame
        """
        options.update(dict(zip(Gui.argnames,args)))
        underride(options, fill=BOTH, expand=1)
        if self.debug:
            override(options, bd=5, relief=RIDGE)

        # create the new frame and push it onto the stack
        frame = self.widget(Frame, **options)
        self.pushfr(frame)
        return frame

    def gr(self, cols, cweights=[1], rweights=[1], **options):
        """create a frame and switch to grid mode.
        cols is the number of columns in the grid (no need to
        specify the number of rows).  cweight and rweight control
        how the widgets expand if the frame expands (see colweights
        and rowweights below).  options are passed along to the frame
        """
        underride(options, fill=BOTH, expand=1)
        fr = self.fr(**options)
        fr.gridding = True
        fr.cols = cols
        fr.i = 0
        fr.j = 0
        self.colweights(cweights)
        self.rowweights(rweights)
        return fr

    def colweights(self, weights):
        """attach weights to the columns of the current grid.
        These weights control how the columns in the grid expand
        when the grid expands.  The default weight is 0, which
        means that the column doesn't expand.  If only one
        column has a value, it gets all the extra space.
        """
        for i, weight in enumerate(weights):
            self.frame.columnconfigure(i, weight=weight)

    def rowweights(self, weights):
        """attach weights to the rows of the current grid.
        These weights control how the rows in the grid expand
        when the grid expands.  The default weight is 0, which
        means that the row doesn't expand.  If only one
        row has a value, it gets all the extra space.
        """
        for i, weight in enumerate(weights):
            self.frame.rowconfigure(i, weight=weight)

    def grid(self, widget, i=None, j=None, **options):
        """pack the given widget in the current grid.
        By default, the widget is packed in the next available
        space, but parameters i and j can override.
        """
        if i == None: i = self.frame.i
        if j == None: j = self.frame.j
        widget.grid(row=i, column=j, **options)

        # increment j by 1, or by columnspan
        # if the widget spans more than one column.
        try:
            incr = options['columnspan']
        except KeyError:
            incr = 1

        self.frame.j += 1
        if self.frame.j == self.frame.cols:
            self.frame.j = 0
            self.frame.i += 1

    # entry
    def en(self, *args, **options):
        """make an entry widget."""
        
        # roll the positional arguments into the option dictionary
        options.update(dict(zip(Gui.argnames,args)))

        # underride fill and expand
        underride(options, fill=BOTH, expand=1)

        # pull the text option out
        text = options.pop('text', '')

        # create the entry and insert the text
        en = self.widget(Entry, **options)
        en.insert(0, text)
        return en

    # canvas
    def ca(self, *args, **options):
        """make a canvas widget."""
        
        underride(options, fill=BOTH, expand=1, sticky=N+S+E+W)
        return self.widget(GuiCanvas, *args, **options)

    # label
    def la(self, *args, **options):
        """make a label widget."""
        
        return self.widget(Label, *args, **options)

    # listbox
    def lb(self, *args, **options):
        """make a listbox."""
        return self.widget(Listbox, *args, **options)

    # button
    def bu(self, *args, **options):
        """make a button"""
        return self.widget(Button, *args, **options)

    # menu button
    def mb(self, *args, **options):
        """make a menubutton"""
        underride(options, relief=RAISED)
        mb = self.widget(Menubutton, *args, **options)
        mb.menu = Menu(mb, tearoff=False)
        mb['menu'] = mb.menu
        return mb

    # menu item
    def mi(self, mb, label='', **options):
        """make a menu item"""
        mb.menu.add_command(label=label, **options)        

    # text entry
    def te(self, *args, **options):
        """make a text entry"""
        return self.widget(Text, *args, **options)

    # scrollbar
    def sb(self, *args, **options):
        """make a text scrollbar"""
        return self.widget(Scrollbar, *args, **options)

    # WARNING: in the following two functions (cb and rb), I
    # attach new attributes to objects (Checkbutton and
    # RadioButton) created by tkinter.  There is no name
    # collision in current versions, but there might be in
    # the future!

    # check button
    def cb(self, *args, **options):
        """make a checkbutton."""
        
        # if the user didn't provide a variable, create one
        try:
            var = options['variable']
        except KeyError:
            var = IntVar()
            override(options, variable=var)
            
        w = self.widget(Checkbutton, *args, **options)
        w.var = var
        return w

    # radio button
    def rb(self, *args, **options):
        """make a radiobutton"""

        w = self.widget(Radiobutton, *args, **options)
        w.var = options['variable']
        w.val = options['value']
        return w

    class ScrollableText:
        """a scrollable text entry is a
        compound widget with a frame that contains a
        text entry on the left and a scrollbar on the right.
        """
        def __init__(self, gui,
                     *args, **options):
            self.frame = gui.fr(*args, **options)
            self.scrollbar = gui.sb(RIGHT, fill=Y)
            self.text = gui.te(LEFT, fill=Y, wrap=WORD,
                                 yscrollcommand=self.scrollbar.set)
            self.scrollbar.configure(command=self.text.yview)
            gui.endfr()

    def st(self, *args, **options):
        """make a scrollable text entry"""
        return self.ScrollableText(self, *args, **options)

    class ScrollableCanvas:
        """a compound widget with a grid that contains a canvas
        and two scrollbars
        """
        def __init__(self, gui, width=200, height=200, **options):
            self.grid = gui.gr(2, **options)
            self.canvas = gui.ca(width=width, height=height, bg='white')

            self.yb = gui.sb(command=self.canvas.yview, sticky=N+S)
            self.xb = gui.sb(command=self.canvas.xview, orient=HORIZONTAL,
                              sticky=E+W)

            self.canvas.configure(xscrollcommand=self.xb.set,
                                  yscrollcommand=self.yb.set,
                                  scrollregion=(0, 0, 400, 400))
            gui.endgr()

    def sc(self, *args, **options):
        """make a scrollable canvas.
        The options provided  apply to the frame only;
        if you want to configure the other widgets, you have to do
        it after invoking st"""
        return self.ScrollableCanvas(self, *args, **options)

    def widget(self, constructor, *args, **options):
        """this is the kernel of the widget constructors.
        (constructor) is the function that will
        be called to build the new widget. (args) is rolled
        into (options), and then (options) is split into widget
        options, pack options and grid options
        """

        # roll the positional arguments into the option dictionary,
        # then divide into options for the widget constructor, pack
        # or grid
        options.update(dict(zip(Gui.argnames,args)))
        widopt, packopt, gridopt = split_options(options)

        # make the widget and either pack or grid it
        widget = constructor(self.frame, **widopt)
        if hasattr(self.frame, 'gridding'):
            self.grid(widget, **gridopt)
        else:
            widget.pack(**packopt)
        return widget


def pop_options(options, names):
    """options is a dictionary; names is a list of keys.
    Remove the given keys from options and
    return a new dictionary with those key-value pairs.
    """
    new = {}
    for name in names:
        if name in options:
            new[name] = options.pop(name)
    return new

def get_options(options, names):
    """options is a dictionary; names is a list of keys.
    return a new dictionary that contains the key-value
    pairs for each key that appears in options.
    """
    new = {}
    for name in names:
        if name in options:
            new[name] = options[name]
    return new

def remove_options(options, names):
    """remove from (options) all the keys in (names)."""
    for name in names:
        if name in options:
            del options[name]

def split_options(options):
    """take a dictionary of options and split it into pack
    options and grid options; anything left is assumed to
    be a widget option
    """
    
    packnames = ['side', 'fill', 'expand', 'anchor',
                 'padx', 'pady', 'ipadx', 'ipady']
    gridnames = ['column', 'columnspan', 'row', 'rowspan',
                 'padx', 'pady', 'ipadx', 'ipady', 'sticky']

    # notice that some options will appear in both packopts
    # and gridopts, so that's why I didn't use pop_options.
    packopts = get_options(options, packnames)
    gridopts = get_options(options, gridnames)

    remove_options(options, packopts)
    remove_options(options, gridopts)

    return options, packopts, gridopts


class BBox(list):
    """a bounding box is a list of coordinates, where each
    coordinate is a list of numbers.  The first pair is the
    upper-left corner; the second pair is the lower-right.

    Creating a new bounding box makes a _shallow_ copy of
    the list of coordinates.  For a deep copy, use Bbox.copy().
    """
    __slots__ = ()

    def copy(self):
        t = [Pos(coord) for coord in bbox]
        return BBox(t)

    # top, bottom, left, and right can be accessed as attributes
    def setleft(bbox, val): bbox[0][0] = val 
    def settop(bbox, val): bbox[0][1] = val 
    def setright(bbox, val): bbox[1][0] = val 
    def setbottom(bbox, val): bbox[1][1] = val 
    
    left = property(lambda bbox: bbox[0][0], setleft)
    top = property(lambda bbox: bbox[0][1], settop)
    right = property(lambda bbox: bbox[1][0], setright)
    bottom = property(lambda bbox: bbox[1][1], setbottom)

    def width(bbox): return bbox.right - bbox.left
    def height(bbox): return bbox.bottom - bbox.top

    # upperleft returns the upper left corner as a new Pos object)
    # lowerright returns the lower right corner.
    def upperleft(bbox): return Pos(bbox[0])
    def lowerright(bbox): return Pos(bbox[1])

    def midright(bbox):
        """return the midpoint of the right edge as a Pos object
        """
        x = bbox.right
        y = (bbox.top + bbox.bottom) / 2.0
        return Pos([x, y])

    def midleft(bbox):
        """return the midpoint of the left edge as a Pos object
        """
        x = bbox.left
        y = (bbox.top + bbox.bottom) / 2.0
        return Pos([x, y])

    def union(self, other):
        """return a new bbox that covers self and other,
        assuming that the positive y direction is UP"""
        left = min(self.left, other.left)
        right = max(self.right, other.right)
        top = max(self.top, other.top)
        bottom = min(self.bottom, other.bottom)
        return BBox([[left, top], [right, bottom]])

    def offset(bbox, pos):
        """return the vector between the upper-left corner of bbox and
        the given position"""
        return Pos([pos[0]-bbox.left, pos[1]-bbox.top])

    def pos(bbox, offset):
        """return the position at the given offset from bbox upper-left"""
        return Pos([offset[0]+bbox.left, offset[1]+bbox.top])

    def flatten(bbox):
        """return a list of four coordinates"""
        return bbox[0] + bbox[1]


class Pos(list):
    """a position is a list of coordinates.

    Because Pos inherits __init__ from list, it makes a copy
    of the argument to the constructor.
    """
    __slots__ = ()

    copy = lambda pos: Pos(pos)

    # x and y can be accessed as attributes
    def setx(pos, val): pos[0] = val 
    def sety(pos, val): pos[1] = val 
    
    x = property(lambda pos: pos[0], setx)
    y = property(lambda pos: pos[1], sety)


# pairiter, pair and flatten are utilities for dealing with
# lists of coordinates

def pairiter(seq):
    """return an iterator that yields consecutive pairs from seq"""
    it = iter(seq)
    while True:
        yield [it.next(), it.next()]

def pair(seq):
    """return a list of consecutive pairs from seq"""
    return [x for x in pairiter(seq)]

def flatten(seq):
    """given a list of lists, return a new list that concatentes
    the elements of (seq).  This just does one level of flattening;
    it is not recursive.
    """
    return sum(seq, [])

# underride and override are utilities for dealing with options
# dictionaries

def underride(d, **kwds):
    """Add entries from (kwds) to (d) only if they are not already set"""
    for key, val in kwds.iteritems():
        if key not in d:
            d[key] = val

def override(d, **kwds):
    """Add entries from (kwds) to (d) even if they are already set"""
    d.update(kwds)


class GuiCanvas(Canvas):
    """this is a wrapper for the Canvas provided by tkinter.
    The primary difference is that it supports coordinate
    transformations, the most common of which is the CanvasTranform,
    which makes canvas coordinates Cartesian (origin in the middle,
    positive y axis going up).

    It also provides methods like circle that provide a
    nice interface to the underlying canvas methods.
    """
    def __init__(self, w, transforms=None, **options):
        Canvas.__init__(self, w, **options)

        # the default transform is a standard CanvasTransform
        if transforms == None:
            self.transforms = [CanvasTransform(self)]
        else:
            self.transforms = transforms

    # the following properties make it possbile to access
    # the width and height of the canvas as if they were attributes
    width = property(lambda self: int(self['width']))
    height = property(lambda self: int(self['height']))

    def add_transform(self, transform, index=None):
        """add the given transform at the given index in the
        transform list (appending is the default).
        """
        if index == None:
            self.transforms.append(transform)
        else:
            self.transforms.insert(index, transform)            
            
    
    def trans(self, coords):
        """apply each of the transforms for this canvas, in order."""
        for trans in self.transforms:
            coords = trans.trans_list(coords)
        return coords

    def invert(self, coords):
        """apply the inverse of each of the transforms, in reverse
        order."""
        t = self.transforms[:]
        t.reverse()
        for trans in t:
            coords = trans.invert_list(coords)
        return coords

    def bbox(self, item):
        """compute the bounding box of the given item
        (transformed from pixel coordinates to transformed
        coordinates).
        """
        if isinstance(item, list):
            item = item[0]
        bbox = Canvas.bbox(self, item)
        if bbox == None: return bbox
        bbox = pair(bbox)
        bbox = self.invert(bbox)
        return BBox(bbox)

    def tcoords(self, item, coords=None):
        """provides get and set access to item coordinates,
        with coordinate translation in both directions.
        """
        if coords != None:
            coords = self.trans(coords)
            coords = flatten(coords)
            Canvas.coords(self, item, *coords)
        else:
            "have to get the coordinates and invert them"
            coords = Canvas.coords(self, item)
            coords = pair(coords)
            coords = self.invert(coords)
            return coords

    # WARNING: the following functions, tmove and flipx, are
    # not thought through very carefully.  The notion of distance
    # can be funny if one of the transforms implements a nonlinear
    # mapping.
        
    def tmove(self, tags, dx=0, dy=0):
        """move all items with the given tags, with dx and dy
        in untransformed coordinates"""
        tags = self.find_withtag(tags)
        for tag in tags:
            coords = self.tcoords(tag)
            for i in range(len(coords)):
                coords[i][0] += dx
                coords[i][1] += dy
            self.tcoords(tag, coords)

    def flipx(self, item):
        """warning: this works in pixel coordinates"""
        coords = Canvas.coords(self, item)
        for i in range(0, len(coords), 2):
            coords[i] *= -1
        Canvas.coords(self, item, *coords)

    # the following are wrappers for the create_ methods
    # inherited from the Canvas class.

    def circle(self, x, y, r, fill='', **options):
        """make a circle with center at (x, y) and radius (r)
        """
        options['fill'] = fill
        coords = self.trans([[x-r, y-r], [x+r, y+r]])
        tag = self.create_oval(coords, options)
        return tag
    
    def oval(self, coords, fill='', **options):
        """make an oval with bounding box (coords) and fill color (fill)
        """
        options['fill'] = fill
        return self.create_oval(self.trans(coords), options)

    def rectangle(self, coords, fill='', **options):
        """make an oval with bounding box (coords) and fill color (fill)
        """
        options['fill'] = fill
        return self.create_rectangle(self.trans(coords), options)

    def line(self, coords, fill='black', **options):
        """make a polyline with vertices at each point in (coords)
        and pen color (fill).
        """
        options['fill'] = fill
        tag = self.create_line(self.trans(coords), options)
        return tag
    
    def polygon(self, coords, fill='', **options):
        """make a close polygon with vertices at each point in (coords)
        and fill color (fill).
        """
        options['fill'] = fill
        return self.create_polygon(self.trans(coords), options)

    def text(self, coord, text='', fill='black', **options):
        """make a text item with the given text and fill color.
        The default anchor is center.
        """
        options['text'] = text
        options['fill'] = fill
        return self.create_text(self.trans([coord]), options)

    def image(self, coord, image, **options):
        """make an image item with the given image at the given position.
        The default anchor is center.
        """
        options['image'] = image
        return self.create_image(self.trans([coord]), options)

    def dump(self, filename='canvas.eps'):
        """create a PostScipt file with the given name and dump
        the contents of the canvas into it.
        """
        bbox = Canvas.bbox(self, ALL)
        if bbox:
            x, y, width, height = bbox
        else:
            x, y, width, height = 0, 0, 100, 100
            
        width -= x
        height -= y
        ps = self.postscript(x=x, y=y, width=width, height=height)
        fp = open(filename, 'w')
        fp.write(ps)
        fp.close()


class Transform:
    """the parent class of transforms, Transform provides methods
    for transforming lists of coordinates.  Subclasses of Transform
    are supposed to implement trans() and invert()
    """
    def trans_list(self, points, func=None):
        """apply (func) to a list of points.
        If (func) is none, apply self.trans.
        """
        if func == None:
            func = self.trans

        if isinstance(points[0], (list, tuple)):
            return [Pos(func(p)) for p in points]
        else:
            return Pos(func(points))

    def invert_list(self, points):
        """apply the inverse transform to the list of points"""
        return self.trans_list(points, self.invert)
    

class CanvasTransform(Transform):
    """under a CanvasTransform, the origin is in the middle of
    the canvas, the positive y-axis is up, and the coordinate
    [1, 1] maps to the point specified by scale.
    """
    def __init__(self, ca, scale=[1, 1]):
        self.ca = ca
        self.scale = scale
    
    def trans(self, p):
        x =  p[0] * self.scale[0] + self.ca.width/2
        y = -p[1] * self.scale[1] + self.ca.height/2      
        return [x, y]

    def invert(self, p):
        x =  (p[0] - self.ca.width/2) / self.scale[0]
        y = - (p[1] - self.ca.height/2) / self.scale[1]
        return [x, y]


class ScaleTransform(Transform):
    """a ScaleTransform scales the coordinates in the x and y directions.
    The origin is half a unit from the upper-left corner; the y axis
    points down.
    """
    def __init__(self, scale=[1, 1]):
        self.scale = scale
    
    def trans(self, p):
        x = p[0] * self.scale[0]
        y = p[1] * self.scale[1]
        return [x, y]

    def invert(self, p):
        x = p[0] / self.scale[0]
        y = p[1] / self.scale[1]
        return [x, y]


class RotateTransform(Transform):
    """rotate the coordinate system
    """
    def __init__(self, theta):
        """rotate the coordinate system (theta) radians counterclockwise.
        """
        self.theta = theta

    def rotate(self, p, theta):
        """rotate the point p counterclockwise (theta) radians and
        return a new point.
        """
        s = sin(theta)
        c = cos(theta)
        x =   c * p[0] + s * p[1]
        y =  -s * p[0] + c * p[1]
        return [x, y]
    
    def trans(self, p):
        return self.rotate(p, self.theta)

    def invert(self, p):
        return self.rotate(p, -self.theta)


class SwirlTransform(RotateTransform):
    """rotate the coordinate system (d) radians counterclockwise,
    where (d) is proportional to the distance from the origin
    """

    def trans(self, p):
        d = sqrt(p[0]*p[0] + p[1]*p[1])
        return self.rotate(p, self.theta*d)

    def invert(self, p):
        d = sqrt(p[0]*p[0] + p[1]*p[1])
        return self.rotate(p, -self.theta*d)


class Callable:
    """this class is used to wrap a function and its arguments
    into an object that can be passed as a callback parameter
    and invoked later.  It is from the Python Cookbook 9.1, page 302
    """
    
    def __init__(self, func, *args, **kwds):
        self.func = func
        self.args = args
        self.kwds = kwds

    def __call__(self):
        return apply(self.func, self.args, self.kwds)

    def __str__(self):
        return self.func.__name__


def tk_example():
    """this example creates a simple GUI using only tkinter
    functions
    """
    tk = Tk()
    
    def hello():
        ca.create_text(100, 100, text='hello', fill='blue')
    
    ca = Canvas(tk, bg='white')
    ca.pack(side=LEFT)

    fr = Frame(tk)
    fr.pack(side=LEFT)

    bu1 = Button(fr, text='Hello', command=hello)
    bu1.pack()
    bu2 = Button(fr, text='Quit', command=tk.quit)
    bu2.pack()
    
    tk.mainloop()

def gui_example():
    """this example creates the same GUI as the previous function,
    but it uses the classes defined in this file
    """
    gui = Gui()

    ca = gui.ca(LEFT, bg='white')
    def hello():
        ca.text([0,0], 'hello', 'blue')
    
    gui.fr(LEFT)
    gui.bu(text='Hello', command=hello)
    gui.bu(text='Quit', command=gui.quit)
    gui.endfr()
    
    gui.mainloop()


def widget_demo():
    """demonstrate a variety of widgets
    """
    g = Gui()

    # FRAME 1

    g.fr(LEFT)

    # a label
    la1 = g.la(TOP, text='This is a label.')

    # an entry
    en = g.en(TOP, fill=NONE)
    en.insert(END, 'This is an entry widget.')

    # another label
    la2 = g.la(TOP, text='')

    def press_me():
        """read the text from the entry and display it as a label
        """
        text = en.get()
        la2.configure(text=text)

    # a button
    bu = g.bu(TOP, text='Press me', command=press_me)

    g.endfr()


    # FRAME 2

    g.fr(LEFT)

    def get_selection():
        """figure out which color is selected in the listbox
        """
        t = lb.curselection()
        try:
            index = int(t[0])
            color = lb.get(index)
            return color
        except:
            return None

    def print_selection(event):
        """print the current color in the listbox
        """
        print get_selection()

    def apply_color():
        """get the current color from the listbox and apply it
        to the circle in the canvas
        """
        color = get_selection()
        if color:
            ca.itemconfig(item1, fill=color)

    # create a frame with a label, listbox and scrollbar
    g.fr(TOP)
    la = g.la(TOP, text='List of colors:')
    lb = g.lb(LEFT, fill=Y)

    # when the user raises the button after selecting a color,
    # print the new selection (if you bind to the button press
    # you get the _previous_ selection)
    lb.bind('<ButtonRelease-1>', print_selection)

    # scrollbar
    sb = g.sb(RIGHT, fill=Y)
    g.endfr()

    # button
    bu = g.bu(BOTTOM, text='Apply color', command=apply_color)
    g.endfr()

    # fill the listbox with color names; if the X11 color list
    # is in the usual place, read it; otherwise use a short list.
    try:
        colors = open('/usr/X11R6/lib/X11/rgb.txt')
        colors.readline()
    except:
        colors = ['\t\t red', '\t\t orange', '\t\t yellow',
                  '\t\t green', '\t\t blue', '\t\t purple']
        
    for line in colors:
        t = line.split('\t')
        name = t[2].strip()
        lb.insert(END, name)

    # tell the listbox and the scrollbar about each other
    lb.configure(yscrollcommand=sb.set)
    sb.configure(command=lb.yview)


    # FRAME 3

    g.fr(LEFT)

    # scrollable canvas
    sc = g.sc()
    ca = sc.canvas

    # make some items
    item1 = ca.circle(0, 0, 70, 'red')
    item2 = ca.rectangle([[0, 0], [60, 60]], 'blue')
    item3 = ca.text([0, 0], 'This is a canvas.', 'white')

    # photo = PhotoImage(file='danger.gif')
    # item4 = ca.create_image(200, 300, image=photo)

    # menubutton
    mb = g.mb(TOP, text='Choose a color')

    def set_color(color):
        ca.itemconfig(item2, fill=color)

    # put some items in the menubutton
    for color in ['red', 'green', 'blue']:
        g.mi(mb, color, command=Callable(set_color, color))

    g.endfr()


    # FRAME 4

    g.fr(LEFT)

    def set_font():
        """get the current settings from the font control widgets
        and configure item3 accordingly
        """
        family = 'helvetica'
        size = fontsize.get()
        weight = b1.var.get()
        slant = b2.var.get()
        font = Font(family=family, size=size, weight=weight, slant=slant)
        print font.actual()
        ca.itemconfig(item3, font=font)

    g.la(TOP, text='Font:')

    # fontsize is the variable associated with the radiobuttons
    fontsize = IntVar()

    # make the radio buttons
    for size in [10, 12, 14, 15, 17, 20]:
        rb = g.rb(TOP, text=str(size), variable=fontsize, value=size,
                  command=set_font)

    # make the check buttons
    b1 = g.cb(TOP, text='Bold', command=set_font, variable=StringVar(),
              onvalue=BOLD, offvalue=NORMAL)
    b1.deselect()
    
    b2 = g.cb(TOP, text='Italic', command=set_font, variable=StringVar(),
              onvalue=ITALIC, offvalue=ROMAN)
    b2.deselect()

    # choose the initial font size
    fontsize.set(10)
    set_font()

    g.endfr()


    # FRAME 5

    g.fr(LEFT)

    # text widget
    te = g.te(TOP, fill=X, height=5, width=40)
    te.insert(END, "This is a Text widget.\n")
    te.insert(END, "It's like a little text editor.\n")
    te.insert(END, "It has more than one line, unlike an Entry widget.\n")

    # scrollable text widget
    st = g.st(TOP)
    st.text.configure(height=5, width=40)
    st.text.insert(END, "This is a Scrollable Text widget.\n")
    st.text.insert(END, "It is defined in Gui.py\n")

    # add some text
    for i in range(100):
        st.text.insert(END, "All work and no play.\n")

    g.endfr()


    # FRAME 6

    # label
    g.la(TOP, text='A grid of buttons:')

    # start a grid with three columns (the weights control how
    # the buttons expand if there is extra space)
    g.gr(3, rweights=[1,1,1], cweights=[1,1,1], side=LEFT)

    def print_num(i):
        print i

    # grid the buttons
    for i in range(1, 10):
        g.bu(text=str(i), sticky=NS, command=Callable(print_num, i))

    g.endgr()

    g.mainloop()


def main(script, function=None, *args):
    if function == None:
        widget_demo()
    else:
        # function is normally tk_example or gui_example
        function = eval(function)
        function()

if __name__ == '__main__':
    main(*sys.argv)




from scitools.std import *
import time

def f(x, m, s):
    return (1.0/(sqrt(2*pi)*s))*exp(-0.5*((x-m)/s)**2)

m = 0
s_start = 2
s_stop = 0.2
s_values = linspace(s_start, s_stop, 30)
x = linspace(m -3*s_start, m + 3*s_start, 1000)

# Make a plot of three s values
for s in s_start, 1, s_stop:
    plot(x, f(x,m,s))
    hold('on')
    legend('s=%s' %s)
title('A Gaussian Bell Function')
savefig('tmp4.eps')

raw_input('Press Return key to quit: ')



#!/usr/bin/env python

"""
Demonstration on how to deal with greek letters and mathematics
in axis labels, titles, legends, and text.
"""
from scitools.std import *

def do_plot(latex='no'):
    tau_1 = linspace(0, 3, 151)
    tau_2 = linspace(0, 3, 11)
    alpha = 1
    theta_1 = tau_1*exp(-alpha*tau_1**2)
    theta_2 = sqrt(tau_2)*exp(-alpha*tau_2**2)
    if latex == 'with $':
        # standard latex text in legends, labels, and title
        plot(tau_1, theta_1, 'r-',
             tau_2, theta_2, 'bo',
             legend=(r'$\theta_1 = \tau e^{-\alpha\tau^2}$',
                     r'$\theta_2 = \sqrt{\tau} e^{-\alpha\tau^2}$'),
             title=r'Plot of $\theta_1$ and $\theta_2$, $\alpha = \sum_{i=1}^n \phi_i$' + ' (%s) ' % latex.replace('$', 'dollar'),
             xlabel=r'$\tau$',
             ylabel=r'$\theta$',
             savefig='tmp_' + backend + '_' + latex.replace(' $', '_dollar') + '.eps')
    elif latex == 'without $':
        # latex math without dollars - some backends will automatically
        # treat this the right way
        plot(tau_1, theta_1, 'r-',
             tau_2, theta_2, 'bo',
             legend=(r'\theta_1 = \tau e^{-\alpha\tau^2}',
                     r'\theta_2 = \sqrt{\tau} e^{-\alpha\tau^2}'),
             title=r'Plot of \theta_1 and \theta_2, \alpha = \sum_{i=1}^n \phi_i' + ' (%s) ' % latex.replace('$', 'dollar'),
             xlabel=r'\tau',
             ylabel=r'\theta',
             savefig='tmp_' + backend + '_' + latex.replace(' $', '_dollar') + '.eps')
    elif latex == 'no':
        # No latex math, just plain words
        plot(tau_1, theta_1, 'r-',
             tau_2, theta_2, 'bo',
             legend=(r'theta_1 = tau exp(-alpha*tau^2)',
                     r'theta_2 = sqrt(tau)*exp(-\alpha*tau^2)'),
             title='Plot of theta_1 and theta_2, alpha = sum_i phi_i' + ' (%s) ' % latex.replace('$', 'dollar'),
             xlabel=r'tau',
             ylabel=r'theta',
             savefig='tmp_' + backend + '_' + latex + '.eps')

print backend
for latex in 'no', 'with $', 'without $':
    figure()
    print 'latex: %s' % latex
    do_plot(latex)
raw_input('Press Return: ')

"""
Summary:

=== gnuplot ===

Plain text (no latex, no dollars) works best. Output in .eps file
will recognize sub and super scripts, and also many greek letters if
the letters are written in latex syntax with an opening backslash.
For .eps hardcopies a little tweaking with inserting some
backslashes if often desirable.

Lables like \tau and \theta get typeset as tab "au" and tab "heta"
due to file communication between python and gnuplot (just skip
the backslashes to avoid this, raw strings do not help - then the
backslash survives).

=== matplotlib ===

Labels and legends work well with and without dollars. The title
should be plain text with embedded dollars and latex as needed.
Real latex text is what is recommended for matplotlib.

"""



"""
Trick to make a nickname for a module!
Can be used to make a nickname py4cs for scitools to keep
backward compatibility with the TCSE3 book.
"""

import sys
import scitools
nickname = 'stcopy'
sys.modules[nickname] = sys.modules['scitools']



"""Test functions for subplot command."""

from easyviztest import *


class test_subplot(EasyvizTestCase):
    def check_subplot_two_rows_one_column(self):
        x,y,v,w = self.get_line_data()
        setp(show=False)
        subplot(2,1,1)
        plot(x,y,'b:')
        title("subplot(2,1,1);plot(..)")
        subplot(2,1,2)
        plot(x,y,'b:',x,v,'r-o')
        title("subplot(2,1,2);plot(..)")
        setp(show=screenplot)
        show()
        n()

    def check_subplot_one_row_two_columns(self):
        x,y,v,w = self.get_line_data()
        setp(show=False)
        subplot(1,2,1)
        plot(x,y,'b:')
        title("subplot(1,2,1);plot(..)")
        subplot(1,2,2)
        plot(x,y,'b:',x,v,'r-o')
        title("subplot(1,2,2);plot(..)")
        setp(show=screenplot)
        show()
        n()

    def check_subplot_two_rows_two_columns(self):
        x,y,v,w = self.get_line_data()
        setp(show=False)
        subplot(2,2,1)
        plot(x,y,'b:')
        title("subplot(2,2,1);plot(..)")
        subplot(2,2,2)
        plot(x,y,'b:',x,v,'r-o')
        title("subplot(2,2,2);plot(..)")
        subplot(2,2,3)
        plot(x,y,'b:',x,v,'r-o',x,w,'c+')
        title("subplot(2,2,3);plot(..)")
        subplot(2,2,4)
        plot(x,y,'b:',x,v,'r-o',x,w,'c+',x,x,'m--')
        title("subplot(2,2,4);plot(..)")
        setp(show=screenplot)
        show()
        n()

    def check_subplot_two_rows_two_columns_mixed(self):
        x,y,v,w = self.get_line_data()
        setp(show=False)
        ax1 = subplot(2,2,1)
        subplot(2,2,3)
        plot(x,y,'b:',x,v,'r-o',x,w,'c+')
        title("subplot(2,2,3);plot(..) (mixed)")
        ax4 = subplot(2,2,4)
        plot(x,y,'b:',x,v,'r-o',x,w,'c+',x,x,'m--')
        setp(ax4,title="subplot(2,2,4);plot(..) (mixed)")
        subplot(2,2,2)
        plot(x,y,'b:',x,v,'r-o')
        title("subplot(2,2,2);plot(..) (mixed)")
        plot(ax1,x,y,'b:')
        title(ax1,"subplot(2,2,1);plot(..) (mixed)")
        setp(show=screenplot)
        show()
        n()

    def check_subplot_31p(self):
        x,y,v,w = self.get_line_data()
        setp(show=False)
        subplot(311)
        plot(x,y,'b:')
        title("subplot(311);plot(..)")
        subplot(312)
        plot(x,y,'b:',x,v,'r-o')
        title("subplot(312);plot(..)")
        subplot(313)
        plot(x,y,'b:',x,v,'r-o',x,w,'c+')
        title("subplot(313);plot(..)")
        setp(show=screenplot)
        show()
        n()

    def check_subplot_23p(self):
        x,y,v,w = self.get_line_data()
        setp(show=False)
        subplot(231)
        plot(x,y,'b:')
        title("subplot(231);plot(..)")
        subplot(232)
        plot(x,y,'b:',x,v,'r-o')
        title("subplot(232);plot(..)")
        subplot(233)
        plot(x,y,'b:',x,v,'r-o',x,w,'c+')
        title("subplot(233);plot(..)")
        subplot(234)
        plot(x,y,'b:',x,v,'r-o',x,w,'c+',x,x,'m--')
        title("subplot(234);plot(..)")
        subplot(235)
        plot(x,y,'b:',x,v,'r-o',x,w,'c+',x,x,'m--',x,-x,'k-.')
        title("subplot(235);plot(..)")
        subplot(236)
        plot(x,y,'b:',x,v,'r-o',x,w,'c+',x,x,'m--',x,-x,'k-.',x,-v,'y2')
        title("subplot(236);plot(..)")
        setp(show=screenplot)
        show()
        n()


def test_suite(level=1):
    suites = []
    if level > 0:
        suites.append(unittest.makeSuite(test_subplot,'check_'))
    total_suite = unittest.TestSuite(suites)
    return total_suite

def test(level=10):
    all_tests = test_suite()
    runner = unittest.TextTestRunner()
    runner.run(all_tests)
    return runner

if __name__ == "__main__":
    test()
    raw_input("press enter to exit")



# -*- coding: utf-8 -*-
"""
=========
Pyreport
=========

Pyreport makes notes out of a python script. It can run the script in a
sandbox and capture its output. It allows for embedding RestructuredText
or LaTeX comments in the code for literate programming and generates a
report made of the literate comments, the code, pretty printed, and the
output of the script (pyreport can capture pylab figures). This is useful
for documentations, making tutorials, but also for sharing python-based
calculations with colleagues.
"""

#*****************************************************************************
#       Copyright (C) 2006-2007 Gaël Varoquaux.
#                                           <gael.varoquaux@normalesup.org>
#
#  Distributed under the terms of the BSD License.
#*****************************************************************************

__author__   = 'Gael Varoquaux'
__license__  = 'BSD'
__revision__ = "$Revision: $"

from version import __version__



import os

from scitools.numpytools import asarray, ones, seq, shape, reshape, meshgrid, \
     NumPyArray, NumPy_type
import scitools.globaldata

def _update_from_config_file(d, section='easyviz'):
    """
    Update the dictionary d with values from the config file scitools.cfg.
    """
    import pprint
    for key in d:
        data = scitools.globaldata._config_data.get(section,{})
        if key in data:
            try:
                d[key] = data[key][0]
            except Exception, e:
                raise Exception('%s, trying to set key=%s to "%s"' % \
                                (str(e), key, data[key][0]))

def _toggle_state(state):
    if state == 'off' or not state:
        return False
    else:
        return True

def _check_type(var, name, type):
    if not isinstance(var, type):
        raise TypeError('variable "%s"=%s is not of %s' % \
                        (name, var, str(type)))
    else:
        return True

def _check_size(a, a_name, expected_size):
    if isinstance(expected_size, int):
        expected_size = (expected_size,) 
    if shape(a) != expected_size:
        raise ValueError('%s has shape %s, expected %s' % \
                         (a_name, a.shape, expected_size))

def _check_xyzv(*args, **kwargs):
    nargs = len(args)
    if nargs == 1:
        x, y, z = [None]*3
        v = asarray(args[0])
    elif nargs == 4:
        x, y, z, v = [asarray(a) for a in args]
    else:
        raise ValueError("_check_xyzv: wrong number of arguments")

    try:
        nx, ny, nz = shape(v)
    except:
        raise ValueError('_check_xyzv: v must be 3D, not %dD' % len(shape(v)))

    indexing = kwargs.get('indexing', 'ij')

    if x is None and y is None and z is None:
        if indexing == 'ij':
            ny, nx = nx, nz  # swap
        x, y, z = meshgrid(range(ny), range(nx), range(nz), indexing=indexing)
    else:
        if indexing == 'ij':
            assert shape(x)==(nx,ny,nz) or shape(x)==(nx,1,1) or \
                   shape(x)==(nx,), \
                   "_check_xyzv: x has shape %s, expected %s, %s, or %s" % \
                   (shape(x), (nx,ny,nz), (nx,1,1), (nx,))
        
            if shape(x) == (nx,ny,nz):
                assert shape(y) == (nx,ny,nz), \
                       "_check_xyzv: x has shape %s, expected y to be %s, " \
                       "not %s" % (shape(x), (nx,ny,nz), shape(y))
                assert shape(z) == (nx,ny,nz), \
                       "_check_xyzv: x has shape %s, expected z to be %s, " \
                       "not %s" % (shape(x), (nx,ny,nz), shape(z))
            elif shape(x) == (nx,1,1):
                assert shape(y) == (1,ny,1), \
                       "_check_xyzv: x has shape %s, expected y to be %s, " \
                       "not %s" % (shape(x), (1,ny,1), shape(y))
                assert shape(z) == (1,1,nz), \
                       "_check_xyzv: x has shape %s, expected z to be %s, " \
                       "not %s" % (shape(x), (1,1,nz), shape(z))
            else: # shape(x) == (nx,)
                assert shape(y) == (ny,), \
                       "_check_xyzv: x has shape %s, expected y to be %s, " \
                       "not %s" % (shape(x), (ny,), shape(y))
                assert shape(z) == (nz,), \
                       "_check_xyzv: x has shape %s, expected z to be %s, " \
                       "not %s" % (shape(x), (nz,), shape(z))
        else:
            assert shape(x)==(nx,ny,nz) or shape(x)==(1,ny,1) or \
                   shape(x)==(ny,), \
                   "_check_xyzv: x has shape %s, expected %s, %s, or %s" % \
                   (shape(x), (nx,ny,nz), (1,ny,1), (ny,))
        
            if shape(x) == (nx,ny,nz):
                assert shape(y) == (nx,ny,nz), \
                       "_check_xyzv: x has shape %s, expected y to be %s, " \
                       "not %s" % (shape(x), (nx,ny,nz), shape(y))
                assert shape(z) == (nx,ny,nz), \
                       "_check_xyzv: x has shape %s, expected z to be %s, " \
                       "not %s" % (shape(x), (nx,ny,nz), shape(z))
            elif shape(x) == (1,ny,1):
                assert shape(y) == (nx,1,1), \
                       "_check_xyzv: x has shape %s, expected y to be %s, " \
                       "not %s" % (shape(x), (nx,1,1), shape(y))
                assert shape(z) == (1,1,nz), \
                       "_check_xyzv: x has shape %s, expected z to be %s, " \
                       "not %s" % (shape(x), (1,1,nz), shape(z))
            else: # shape(x) == (ny,)
                assert shape(y) == (nx,), \
                       "_check_xyzv: x has shape %s, expected y to be %s, " \
                       "not %s" % (shape(x), (nx,), shape(y))
                assert shape(z) == (nz,), \
                       "_check_xyzv: x has shape %s, expected z to be %s, " \
                       "not %s" % (shape(x), (nz,), shape(z))
        
    return x, y, z, v

def _check_xyz(*args, **kwargs):
    nargs = len(args)
    if nargs == 1:
        x, y = [None]*2
        z = asarray(args[0])
    elif nargs == 3:
        x, y, z = [asarray(a) for a in args]
    else:
        raise TypeError("_check_xyz: wrong number of arguments")
    
    try:
        nx, ny = shape(z)
    except:
        raise ValueError("z must be 2D, not %dD" % len(shape(z)))

    indexing = kwargs.get('indexing', 'ij')

    if x is None and y is None:
        if indexing == 'ij':
            nx, ny = ny, nx  # swap
        x, y = meshgrid(range(ny), range(nx), indexing=indexing)
    else:
        if indexing == 'ij':
            assert shape(x) == (nx,ny) or shape(x) == (nx,1) or len(x) == nx, \
                   "_check_xyz: x has shape %s, expected %s, %s, or %s" % \
                   (shape(x), (nx,ny), (nx,1), (nx,))
        
            assert shape(y) == (nx,ny) or shape(y) == (1,ny) or len(y) == ny, \
                   "_check_xyz: y has shape %s, expected %s, %s, or %s" % \
                   (shape(y), (nx,ny), (1,ny), (ny,))
        else:
            assert shape(x) == (nx,ny) or shape(x) == (1,ny) or len(x) == ny, \
                   "_check_xyz: x has shape %s, expected %s, %s, or %s" % \
                   (shape(x), (nx,ny), (1,ny), (ny,))
        
            assert shape(y) == (nx,ny) or shape(y) == (nx,1) or len(y) == nx, \
                   "_check_xyz: y has shape %s, expected %s, %s, or %s" % \
                   (shape(y), (nx,ny), (nx,1), (nx,))
        
    return x, y, z
    
def _check_xyuv(*args, **kwargs):
    nargs = len(args)
    if nargs == 2:
        x, y = [None]*2
        u, v = [asarray(a) for a in args]
    elif nargs == 4:
        x, y, u, v = [asarray(a) for a in args]
    else:
        raise TypeError("_check_xyuv: wrong number of arguments")
    
    indexing = kwargs.get('indexing', 'ij')

    us = shape(u)
    assert us == shape(v), "_check_xyuv: u and v must be of same shape"
    
    if len(us) == 1:
        if x is None and y is None:
            x = range(us[0])
            y = range(us[0])
        else:
            assert shape(x) == us, \
                   "_check_xyuv: x has shape %s, expected %s" % (shape(x), us)
            assert shape(y) == us, \
                   "_check_xyuv: y has shape %s, expected %s" % (shape(y), us)
    elif len(us) == 2:
        nx, ny = us
        if x is None and y is None:
            if indexing == 'ij':
                x = seq(nx-1)
                y = seq(ny-1)
            else:
                x = seq(ny-1)
                y = seq(nx-1)                
        else:
            if indexing == 'ij':
                assert shape(x)==(nx,ny) or shape(x)==(nx,1) or \
                       shape(x)==(nx,), \
                       "_check_xyuv: x has shape %s, expected %s, %s, " \
                       "or %s" % (shape(x), (nx,ny), (nx,1), (nx,))
                assert shape(y)==(nx,ny) or shape(y)==(1,ny) or \
                       shape(y)==(ny,), \
                       "_check_xyuv: y has shape %s, expected %s, %s, " \
                       "or %s" % (shape(y), (nx,ny), (1,ny), (ny,))
            else:
                assert shape(x)==(nx,ny) or shape(x)==(1,ny) or \
                       shape(x)==(ny,), \
                       "_check_xyuv: x has shape %s, expected %s, %s, " \
                       "or %s" % (shape(x), (nx,ny), (1,ny), (ny,))
                assert shape(y)==(nx,ny) or shape(y)==(nx,1) or \
                       shape(y)==(nx,), \
                       "_check_xyuv: y has shape %s, expected %s, %s, " \
                       "or %s" % (shape(y), (nx,ny), (nx,1), (nx,))
    else:
        raise ValueError("_check_xyuv: u must be 1D or 2D, not %dD" % len(us))
        
    return x, y, u, v

def _check_xyzuvw(*args, **kwargs):
    nargs = len(args)
    if nargs == 4:
        x, y = [None]*2
        z, u, v, w = [asarray(a) for a in args]
    elif nargs == 6:
        x, y, z, u, v, w = [asarray(a) for a in args]
    else:
        raise TypeError("_check_xyzuvw: wrong number of arguments")

    indexing = kwargs.get('indexing', 'xy')

    us = shape(u)
    assert us == shape(v) == shape(w), \
           "_check_xyzuvw: u, v, and w must be of same shape"
    
    if len(us) == 1:
        if x is None and y is None:
            x = seq(us[0]-1)
            y = seq(us[0]-1)
        else:
            assert shape(x) == us, \
                   "_check_xyuv: x has shape %s, expected %s" % (shape(x), us)
            assert shape(y) == us, \
                   "_check_xyuv: y has shape %s, expected %s" % (shape(y), us)
        assert shape(z) == us, \
               "_check_xyuv: z has shape %s, expected %s" % (shape(z), us)
    elif len(us) == 2:
        nx, ny = us
        if x is None and y is None:
            x, y, z = _check_xyz(z, indexing=indexing)
        else:
            x, y, z = _check_xyz(x, y, z, indexing=indexing)
        assert shape(z) == us, \
               "_check_xyzuvw: z, u, v, and w must be of same shape"
    elif len(us) == 3:
        nx, ny, nz = us
        if x is None and y is None:
            if indexing == 'ij':
                nx, ny = ny, nx  # swap
            x, y, junk = meshgrid(seq(ny-1), seq(nx-1), seq(nz-1))
        else:
            if indexing == 'ij':
                assert shape(x)==us or shape(x)==(nx,1,1) or shape(x)==(nx,), \
                       "_check_xyzuvw: x has shape %s, expected %s, %s, or %s"\
                       % (shape(x), us, (nx,1,1), (nx,))
                assert shape(y)==us or shape(y)==(1,ny,1) or shape(y)==(ny,), \
                       "_check_xyzuvw: y has shape %s, expected %s, %s, or %s"\
                       % (shape(y), us, (1,ny,1), (ny,))
            else:
                assert shape(x)==us or shape(x)==(1,ny,1) or shape(x)==(ny,), \
                       "_check_xyzuvw: x has shape %s, expected %s, %s, or %s"\
                       % (shape(x), us, (1,ny,1), (ny,))
                assert shape(y)==us or shape(y)==(nx,1,1) or shape(y)==(nx,), \
                       "_check_xyzuvw: y has shape %s, expected %s, %s, or %s"\
                       % (shape(y), us, (nx,1,1), (nx,))                
        assert shape(z) == us or shape(z) == (1,1,nz) or shape(z) == (nz,), \
               "_check_xyzuvw: z has shape %s, expected %s, %s, or %s" % \
               (shape(z), us, (1,1,nz), (nz,))
    else:
        raise ValueError(
            "_check_xyzuvw: u must be 1D, 2D, or 3D, not %dD" % len(us))

    return x, y, z, u, v, w

def arrayconverter(a):
    """Convert the numpy array a with type 'numpy.ndarray' into a
    Numeric.array object (type 'array'). This is useful when a
    'numpy.ndarray' object is not accepted, which is the case in the
    gnuplot module.
    """
    if NumPy_type(a) == 'numpy':
        import Numeric
        return Numeric.array(a.tolist())
    else:
        return a

def _cmpPlotProperties(a,b):
    """Sort cmp-function for PlotProperties"""
    plotorder = [Volume, Streams, Surface, Contours,
                 VelocityVectors, Bars, Line] 
    assert isinstance(a, PlotProperties)
    assert isinstance(b, PlotProperties)
    assert len(PlotProperties.__class__.__subclasses__(PlotProperties)) == \
               len(plotorder) # Check all subclasses is in plotorder
    return cmp(plotorder.index(a.__class__),plotorder.index(b.__class__))



#!/usr/bin/env python
"""
Class for a scalar (or vector) field over a BoxGrid or UniformBoxGrid grid.
"""

from scitools.BoxGrid import BoxGrid, UniformBoxGrid, X, Y, Z
from numpy import zeros, array, transpose

__all__ = ['BoxField', 'BoxGrid', 'UniformBoxGrid', 'X', 'Y', 'Z',
           'dolfin_function2BoxField', 'update_from_dolfin_array']


class Field(object):
    """
    General base class for all grids. Holds a connection to a
    grid, a name of the field, and optionally a list of the
    independent variables and a string with a description of the
    field.
    """
    def __init__(self, grid, name,
                 independent_variables=None,
                 description=None,
                 **kwargs):
        self.grid = grid

        self.name = name
        self.independent_variables = independent_variables
        if self.independent_variables is None:
            # copy grid.dirnames as independent variables:
            self.independent_variables = self.grid.dirnames

        # metainformation:
        self.meta = {'description': description,}
        self.meta.update(kwargs)  # user can add more meta information


class BoxField(Field):
    """
    Field over a BoxGrid or UniformBoxGrid grid.

    =============      =============================================
      Attributes                       Description
    =============      =============================================
    grid               reference to the underlying grid instance
    values             array holding field values at the grid points
    =============      =============================================

    """
    def __init__(self, grid, name, vector=0, **kwargs):
        """
        Initialize scalar or vector field over a BoxGrid/UniformBoxGrid.

        =============      ===============================================
          Arguments                          Description
        =============      ===============================================
        *grid*             grid instance
        *name*             name of the field
        *vector*           scalar field if 0, otherwise the no of vector
                           components (spatial dimensions of vector field)
        *values*           (*kwargs*) optional array with field values
        =============      ===============================================

        Here is an example::

        >>> g = UniformBoxGrid(min=[0,0], max=[1.,1.], division=[3, 4])

        >>> print g
        domain=[0,1]x[0,1]  indices=[0:3]x[0:4]

        >>> u = BoxField(g, 'u')
        >>> u.values = u.grid.vectorized_eval(lambda x,y: x + y)

        >>> i = 1; j = 2
        >>> print 'u(%g, %g)=%g' % (g.coor[X][i], g.coor[Y][j], u.values[i,j])
        u(0.333333, 0.5)=0.833333

        >>> # visualize:
        >>> from scitools.std import surf
        >>> surf(u.grid.coorv[X], u.grid.coorv[Y], u.values)

        ``u.grid.coorv`` is a list of coordinate arrays that are
        suitable for Matlab-style visualization of 2D scalar fields.
        Also note how one can access the coordinates and u value at
        a point (i,j) in the grid.
        """
        Field.__init__(self, grid, name, **kwargs)

        if vector > 0:
            # for a vector field we add a "dimension" in values for
            # the various vector components (first index):
            self.required_shape = [vector]
            self.required_shape += list(self.grid.shape)
        else:
            self.required_shape = self.grid.shape

        if 'values' in kwargs:
            values = kwargs['values']
            self.set_values(values)
        else:
            # create array of scalar field grid point values:
            self.values = zeros(self.required_shape)

        # doesn't  work: self.__getitem__ = self.values.__getitem__
        #self.__setitem__ = self.values.__setitem__

    def copy_values(self, values):
        """Take a copy of the values array and reshape it if necessary."""
        self.set_values(values.copy())

    def set_values(self, values):
        """Attach the values array to this BoxField object."""
        if values.shape == self.required_shape:
            self.values = values  # field data are provided
        else:
            try:
                values.shape = self.required_shape
                self.values = values
            except ValueError:
                raise ValueError(
                    'values array are incompatible with grid size; '\
                    'shape is %s while required shape is %s' % \
                    (values.shape, self.required_shape))

    def update(self):
        """Update the self.values array (if grid has been changed)."""
        if self.grid.shape != self.values.shape:
            self.values = zeros(self.grid.shape)

    # these are slower than u_ = u.values; u_[i] since an additional
    # function call is required compared to NumPy array indexing:
    def __getitem__(self, i):  return self.values[i]
    def __setitem__(self, i, v):  self.values[i] = v

    def __str__(self):
        if len(self.values.shape) > self.grid.nsd:
            s = 'Vector field with %d components' % self.values.shape[-1]
        else:
            s = 'Scalar field'
        s += ', over ' + str(self.grid)
        return s

    def gridline(self, start_coor, direction=0, end_coor=None,
                 snap=True):
        """
        Return a coordinate array and corresponding field values
        along a line starting with start_coor, in the given
        direction, and ending in end_coor (default: grid boundary).
        Two more boolean values are also returned: fixed_coor
        (the value of the fixed coordinates, which may be different
        from those in start_coor if snap=True) and snapped (True if
        the line really had to be snapped onto the grid, i.e.,
        fixed_coor differs from coordinates in start_coor.

        If snap is True, the line is snapped onto the grid, otherwise
        values along the line must be interpolated (not yet implemented).

        >>> g2 = UniformBoxGrid.init_fromstring('[-1,1]x[-1,2] [0:3]x[0:4]')
        >>> print g2
        UniformBoxGrid(min=[-1. -1.], max=[ 1.  2.],
        division=[3, 4], dirnames=('x', 'y'))
        >>> print g2.coor
        [array([-1.        , -0.33333333,  0.33333333,  1.        ]),
        array([-1.  , -0.25,  0.5 ,  1.25,  2.  ])]

        >>> u = BoxField(g2, 'u')
        >>> u.values = u.grid.vectorized_eval(lambda x,y: x + y)
        >>> xc, uc, fixed_coor, snapped = u.gridline((-1,0.5), 0)
        >>> print xc
        [-1.         -0.33333333  0.33333333  1.        ]
        >>> print uc
        [-0.5         0.16666667  0.83333333  1.5       ]
        >>> print fixed_coor, snapped
        [0.5] False
        >>> #plot(xc, uc, title='u(x, y=%g)' % fixed_coor)
        """
        if not snap:
            raise NotImplementedError('Use snap=True, no interpolation impl.')

        slice_index, snapped = \
             self.grid.gridline_slice(start_coor, direction, end_coor)
        fixed_coor = [self.grid[s][i] for s,i in enumerate(slice_index) \
                      if not isinstance(i, slice)]
        if len(fixed_coor) == 1:
            fixed_coor = fixed_coor[0]  # avoid returning list of length 1
        return self.grid.coor[direction][slice_index[direction].start:\
                                         slice_index[direction].stop], \
               self.values[slice_index], fixed_coor, snapped

    def gridplane(self, value, constant_coor=0, snap=True):
        """
        Return two one-dimensional coordinate arrays and
        corresponding field values over a plane where one coordinate,
        constant_coor, is fixed at a value.

        If snap is True, the plane is snapped onto a grid plane such
        that the points in the plane coincide with the grid points.
        Otherwise, the returned values must be interpolated (not yet impl.).
        """
        if not snap:
            raise NotImplementedError('Use snap=True, no interpolation impl.')

        slice_index, snapped = self.grid.gridplane_slice(value, constant_coor)
        if constant_coor == 0:
            x = self.grid.coor[1]
            y = self.grid.coor[2]
        elif constant_coor == 1:
            x = self.grid.coor[0]
            y = self.grid.coor[2]
        elif constant_coor == 2:
            x = self.grid.coor[0]
            y = self.grid.coor[1]
        fixed_coor = self.grid.coor[constant_coor][slice_index[constant_coor]]
        return x, y, self.values[slice_index], fixed_coor, snapped

def _rank12rankd_mesh(a, shape):
    """
    Given rank 1 array a with values in a mesh with the no of points
    described by shape, transform the array to the right "mesh array"
    with the same shape.
    """
    shape = list(shape)
    shape.reverse()
    if len(a.shape) == 1:
        return a.reshape(shape).transpose()
    else:
        raise ValueError('array a cannot be multi-dimensional (not %s), ' \
                         'break it up into one-dimensional components' \
                         % a.shape)

def dolfin_mesh2UniformBoxGrid(dolfin_mesh, division=None):
    """
    Turn a regular, structured DOLFIN finite element mesh into
    a UniformBoxGrid object. (Application: plotting with scitools.)
    Standard DOLFIN numbering numbers the nodes along the x[0] axis,
    then x[1] axis, and so on.
    """
    if hasattr(dolfin_mesh, 'structured_data'):
        coor = dolfin_mesh.structured_data
        min_coor = [c[0]  for c in coor]
        max_coor = [c[-1] for c in coor]
        division = [len(c)-1 for c in coor]
    else:
        if division is None:
            raise ValueError('division must be given when dolfin_mesh does not have a strutured_data attribute')
        else:
            coor = dolfin_mesh.coordinates() # numpy array
            min_coor = coor[0]
            max_coor = coor[-1]

    return UniformBoxGrid(min=min_coor, max=max_coor,
                          division=division)

def dolfin_mesh2BoxGrid(dolfin_mesh, division=None):
    """
    Turn a structured DOLFIN finite element mesh into
    a BoxGrid object. (Mostly for ease of plotting with scitools.)
    Standard DOLFIN numbering numbers the nodes along the x[0] axis,
    then x[1] axis, and so on.
    """
    if hasattr(dolfin_mesh, 'structured_data'):
        coor = dolfin_mesh.structured_data
        return BoxGrid(coor)
    else:
        if division is None:
            raise ValueError('division must be given when dolfin_mesh does not have a strutured_data attribute')
        else:
            c = dolfin_mesh.coordinates() # numpy array
            shape = [n+1 for n in division]  # shape for points in each dir.

            c2 = [c[:,i] for i in range(c.shape[1])]  # split x,y,z components
            for i in range(c.shape[1]):
                c2[i] = _rank12rankd_mesh(c2[i], shape)
            # extract coordinates in the different directions
            coor = []
            if len(c2) == 1:
                coor = [c2[0][:]]
            elif len(c2) == 2:
                coor = [c2[0][:,0], c2[1][0,:]]
            elif len(c2) == 3:
                coor = [c2[0][:,0,0], c2[1][0,:,0], c2[2][0,0,:]]
            return BoxGrid(coor)


def dolfin_function2BoxField(dolfin_function, dolfin_mesh,
                             division=None, uniform_mesh=True):
    """
    Turn a DOLFIN P1 finite element field over a structured mesh into
    a BoxField object. (Mostly for ease of plotting with scitools.)
    Standard DOLFIN numbering numbers the nodes along the x[0] axis,
    then x[1] axis, and so on.

    If the DOLFIN function employs elements of degree > 1, one should
    project or interpolate the field onto a field with elements of
    degree=1.
    """
    if dolfin_function.ufl_element().degree() != 1:
        raise TypeError("""\
The dolfin_function2BoxField function works with degree=1 elements
only. The DOLFIN function (dolfin_function) has finite elements of type
%s
i.e., the degree=%d != 1. Project or interpolate this function
onto a space of P1 elements, i.e.,

V2 = FunctionSpace(mesh, 'CG', 1)
u2 = project(u, V2)
# or
u2 = interpolate(u, V2)

""" % (str(dolfin_function.ufl_element()), dolfin_function.ufl_element().degree()))

    nodal_values = dolfin_function.vector().array().copy()

    if uniform_mesh:
        grid = dolfin_mesh2UniformBoxGrid(dolfin_mesh, division)
    else:
        grid = dolfin_mesh2BoxGrid(dolfin_mesh, division)

    if nodal_values.size > grid.npoints:
        # vector field, treat each component separately
        ncomponents = int(nodal_values.size/grid.npoints)
        try:
            nodal_values.shape = (ncomponents, grid.npoints)
        except ValueError, e:
            raise ValueError('Vector field (nodal_values) has length %d, there are %d grid points, and this does not match with %d components' % (nodal_values.size, grid.npoints, ncomponents))
        vector_field = [_rank12rankd_mesh(nodal_values[i,:].copy(),
                                          grid.shape) \
                        for i in range(ncomponents)]
        nodal_values = array(vector_field)
        bf = BoxField(grid, name=dolfin_function.name(),
                      vector=ncomponents, values=nodal_values)
    else:
        try:
            nodal_values = _rank12rankd_mesh(nodal_values, grid.shape)
        except ValueError, e:
            raise ValueError('DOLFIN function has vector of size %s while the provided mesh has %d points and shape %s' % (nodal_values.size, grid.npoints, grid.shape))
        bf = BoxField(grid, name=dolfin_function.name(),
                      vector=0, values=nodal_values)
    return bf

def update_from_dolfin_array(dolfin_array, box_field):
    """
    Update the values in a BoxField object box_field with a new
    DOLFIN array (dolfin_array). The array must be reshaped and
    transposed in the right way
    (therefore box_field.copy_values(dolfin_array) will not work).
    """
    nodal_values = dolfin_array.copy()
    if len(nodal_values.shape) > 1:
        raise NotImplementedError # no support for vector valued functions yet
                                  # the problem is in _rank12rankd_mesh
    try:
        nodal_values = _rank12rankd_mesh(nodal_values, box_field.grid.shape)
    except ValueError, e:
        raise ValueError('DOLFIN function has vector of size %s while the provided mesh demands %s' % (nodal_values.size, grid.shape))
    box_field.set_values(nodal_values)
    return box_field

def _test(g):
    print 'grid: %s' % g

    # function: 1 + x + y + z
    def f(*args):
        sum = 1.0
        for x in args:
            sum = sum + x
        return sum

    u = BoxField(g, 'u')
    v = BoxField(g, 'v', vector=g.nsd)

    u.values = u.grid.vectorized_eval(f)  # fill field values

    if   g.nsd == 1:
        v[:,X] = u.values + 1  # 1st component
    elif g.nsd == 2:
        v[:,:,X] = u.values + 1  # 1st component
        v[:,:,Y] = u.values + 2  # 2nd component
    elif g.nsd == 3:
        v[:,:,:,X] = u.values + 1  # 1st component
        v[:,:,:,Y] = u.values + 2  # 2nd component
        v[:,:,:,Z] = u.values + 3  # 3rd component

    # write out field values at the mid point of the grid
    # (convert g.shape to NumPy array and divide by 2 to find
    # approximately the mid point)
    midptindex = tuple(array(g.shape,int)/2)
    ptcoor = g[midptindex]
    # tuples with just one item does not work as indices
    print 'mid point %s has indices %s' % (ptcoor, midptindex)
    print 'f%s=%g' % (ptcoor, f(*ptcoor))
    print 'u at %s: %g' % (midptindex, u[midptindex])
    v_index = list(midptindex); v_index.append(slice(g.nsd))
    print 'v at %s: %s' % (midptindex, v[v_index])

    # test extraction of lines:
    if u.grid.nsd >= 2:
        direction = u.grid.nsd-1
        coor, u_coor, fixed_coor, snapped = \
              u.gridline(u.grid.min_coor, direction)
        if snapped: print 'Error: snapped line'
        print 'line in x[%d]-direction, starting at %s' % \
              (direction, u.grid.min_coor)
        print coor
        print u_coor

        direction = 0
        point = u.grid.min_coor.copy()
        point[direction+1] = u.grid.max_coor[direction+1]
        coor, u_coor, fixed_coor, snapped = \
              u.gridline(u.grid.min_coor, direction)
        if snapped: print 'Error: snapped line'
        print 'line in x[%d]-direction, starting at %s' % \
              (direction, point)
        print coor
        print u_coor

    if u.grid.nsd == 3:
        y_center = (u.grid.max_coor[1] + u.grid.min_coor[1])/2.0
        xc, yc, uc, fixed_coor, snapped = \
            u.gridplane(value=y_center, constant_coor=1)
        print 'Plane y=%g:' % fixed_coor,
        if snapped: print ' (snapped from y=%g)' % y_center
        else: print
        print xc
        print yc
        print uc

def _test2():
    g1 = UniformBoxGrid(min=0, max=1, division=4)
    _test(g1)
    spec = '[0,1]x[-1,2] with indices [0:4]x[0:3]'
    g2 = UniformBoxGrid.init_fromstring(spec)
    _test(g2)
    g3 = UniformBoxGrid(min=(0,0,-1), max=(1,1,1), division=(4,3,2))
    _test(g3)

if __name__ == '__main__':
    _test2()



#!/usr/bin/env python
"""
Install scitools with easyviz

Usage:

python setup.py install [, --prefix=$PREFIX --easyviz_backend backendname]
"""

import os, sys, socket, re, glob, platform

scripts = [os.path.join("bin", "scitools"), os.path.join("bin", "pyreport")]

if platform.system() == "Windows" or "bdist_wininst" in sys.argv:
    # In the Windows command prompt we can't execute Python scripts 
    # without a .py extension. A solution is to create batch files
    # that runs the different scripts.
    batch_files = []
    for script in scripts:
        batch_file = script + ".bat"
        f = open(batch_file, "w")
        f.write('python "%%~dp0\%s" %%*\n' % os.path.split(script)[1])
        f.close()
        batch_files.append(batch_file)
    scripts.extend(batch_files)

# NOTE: now we force matplotlib as default backend if it is installed:
# (previously gnuplot was always default)
try:
    import matplotlib
    default_easyviz_backend = 'matplotlib'
except:  # cannot accept any error here
    try:
        import Gnuplot
        default_easyviz_backend = 'gnuplot'
    except:
        # Neither Gnuplot nor Matplotlib is installed. Matplotlib is now set
        # as default backend for Easyviz.
        default_easyviz_backend = 'matplotlib'

# modify config file so that it sets the wanted backend for easyviz
config_file = os.path.join('lib', 'scitools', 'scitools.cfg')
if '--easyviz_backend' in sys.argv:
    try:
        i = sys.argv.index('--easyviz_backend')
        default_easyviz_backend = \
            sys.argv[i+1]
        del sys.argv[i:i+2]
    except IndexError:
        print '--easyviz_backend must be followed by a name like '\
            '\ngnuplot, matplotlib, etc.'
        sys.exit(1)
print 'default scitools.easyviz backend becomes', default_easyviz_backend
print '(could be set by the --easyviz_backend option to setup.py)\n'
# write new config file and change backend line
os.rename(config_file, config_file + '.old~~')
infile = open(config_file + '.old~~', 'r')
outfile = open(config_file, 'w')
for line in infile:
    if line.lstrip().startswith('['):
        section = line.lower().strip()[1:-1]
    if line.lstrip().startswith('backend') and section == 'easyviz':
        outfile.write('backend     = %s  ; default backend\n' % \
                      default_easyviz_backend)
    else:
        outfile.write(line)
infile.close();  outfile.close()

if  __file__ == 'setupegg.py':
    # http://peak.telecommunity.com/DevCenter/setuptools
    from setuptools import setup, Extension
else:
    from distutils.core import setup

# make sure we import from scitools in this package, not an installed one:
sys.path.insert(0, os.path.join('lib')); import scitools

setup(
    version = str(scitools.version), 
    author = ', '.join(scitools.author),
    author_email = "<hpl@simula.no>",
    description = scitools.__doc__,
    license = "BSD",
    name = "SciTools",
    url = "http://scitools.googlecode.com",
    package_dir = {'': 'lib'},
    packages = ["scitools",
                os.path.join("scitools", "easyviz"), 
		],
    package_data = {'': ['scitools.cfg']},
    scripts = scripts,
    data_files=[(os.path.join("share", "man", "man1"),
                 [os.path.join("doc", "man", "man1", "scitools.1.gz"),
                  os.path.join("doc", "man", "man1", "pyreport.1.gz")])],
    )

if os.path.isfile(config_file + '.cop'):
    os.rename(config_file + '.cop', config_file)




"""
Import modules and definitions for the scitools library.

Import of Numerical Python
--------------------------

  - If scipy is installed, a from scipy import * is executed.
    scipy requires numpy as basic array library, and scipy replaces
    numpy functionality (at some cost of speed since scipy functions
    are more general).

    The user can turn off import of scipy in the configuration file
    (see below).

  - If scipy is not available or not wanted, two options are possible,
    depending on what is set in the "numpy" section of the configuration
    file:

      1. from numpy import *
         (configuration file: numpytools = no)

      2. from numpytools import *
         which allows the user to transparently use Numeric, numarray,
         and numpy
         (configuration file: numpytools = yes)

To make numpy/scipy and math more similar (and thereby ease
vectorization of user-defined functions), we introduce the math
function names asin, acos, and atan for numpy/scipy's arcsin,
arccos, and arctan.


Other modules
-------------

The from scitools.basics import * statement performs a
from numpy (or scipy) import * as explained above, plus an import
of the modules os, sys, operator, and math. The modules
StringFunction and glob are also imported.


Definition of variables
-----------------------

  - SciTools defines a series of variables to monitor execution of
    programs: SAFECODE, OPTIMIZATION, backend (for Easyviz),
    VERBOSE, DEBUG, has_scipy. Many of these are set in a configuration
    file (see below) or in environment variables (SCITOOLS_SAFECODE,
    SCITOOLS_VERBOSE, SCITOOLS, SCITOOLS_DEBUG, SCITOOLS_easyviz_backend).

All these variables are contained in basics.py.


Debug functions
---------------

The following functions are imported from the debug module:

  - watch(var): print out the name, type, and value of a variable and
    where in a program this output was requested (used to monitor variables).

  - trace(message): print a message and where in the program this
    message was requested (used to trace the execution).

These debug functions require the DEBUG variable to be different from 0.


Default configuration file
--------------------------
"""
# Collect all import statements done by this module
_import_list = []
# Measure CPU time for various import statements
_import_times = 'scitools import times: '
import time as _time
_t0 = _time.clock()

import os
# Add the configuration file as part of the doc string
__doc__ += open(os.path.join(os.path.dirname(__file__), 'scitools.cfg')).read()

# Load configuration file through import of the globaldata module
from globaldata import *   # read-only import of global variables
_import_list.append("from globaldata import *")
import globaldata as _globaldata
#import pprint; pprint.pprint(_globaldata._config_data)

_t1 = _time.clock(); _import_times += 'config=%g ' % (_t1 - _t0)

# *** try to import SciPy if it is installed ***
# idea: load scipy, if not, load numpy or numpytools

has_scipy = False   # indicates for all application scripts if one has scipy
if _globaldata._load_scipy:
    try:
        from numpyutils import *   # loads numpy too
        from math import factorial # override
        _import_list.append("from numpy import *")
        _import_list.append("from numpyutils import *")
        from scipy import *        # overrides some numpy functions
        has_scipy = True
        from numpy.lib.scimath import *    # will be part of scipy import
        del _import_list[-2]       # del the numpy entry (scipy overrides)
        _import_list.append("from scipy import *")
    except ImportError:
        # ImportError is likely due to missin scipy
        if VERBOSE >= 2: print 'tried to import scipy, but could not find it'
        pass
    _t2 = _time.clock(); _import_times += 'scipy=%g ' % (_t2 - _t1)

# didn't want, or couldn't load, scipy:
if not has_scipy:
    if _globaldata._load_numpytools:
        from numpytools import *
        _t2 = _time.clock(); _import_times += 'numpytools=%g ' % (_t2 - _t1)
        _import_list.append("from numpytools import *")
    else:
        # load numpy and numpyutils
        try:
            from numpyutils import *   # loads numpy too
            _import_list.append("from numpy import *\nfrom numpyutils import *")
            from numpy.lib.scimath import *   # more general sin, cos etc

        except ImportError:
            raise ImportError('numpy was requested, but it could not be found')
        _t2 = _time.clock(); _import_times += 'numpy=%g ' % (_t2 - _t1)

# nice to have imports:
import sys, operator, math
from scitools.StringFunction import StringFunction
from glob import glob
_import_list.append("import sys, operator, math")
_import_list.append("from StringFunction import StringFunction")
_import_list.append("from glob import glob")

# nice to have symbols (for math/numpy/scipy equivalence/vectorization):
asin = arcsin
acos = arccos
atan = arctan

import debug
debug.DEBUG = DEBUG
from debug import watch, trace
_import_list.append("from debug import watch, trace")

if VERBOSE >= 2:
    for i in _import_list:
        print i

if VERBOSE >= 3:
    print _import_times

__doc__ += '\nImport statements in this module:\n' + '\n'.join(_import_list)



#!/usr/bin/env python

from scitools.std import *

from numpy import MachAr
eps = MachAr().eps

mesh(peaks())
title('Wireframe mesh of the peaks function')

#hardcopy('mesh1a.eps', color=True)
#hardcopy('mesh1a.png', color=True)

figure()
x = linspace(-8,8,31)
xv, yv = ndgrid(x,x)
r = sqrt(xv**2 + yv**2) + eps
values = sin(r)/r
mesh(xv, yv, values, xlabel='x', ylabel='y', zlabel='z',
     title='f(x,y)=sqrt(x**2+y**2)/(x**2+y**2)')

#hardcopy('mesh1b.eps', color=True)
#hardcopy('mesh1b.png', color=True)

figure()
x = y = linspace(0,pi,32)
xv, yv = ndgrid(x,y)
values = sin(yv**2 + xv) - cos(yv - xv**2)
setp(interactive=False)
subplot(221);  mesh(values);
subplot(222);  mesh(x, y, values, hidden='off')
subplot(223);  mesh(xv, yv, values)
subplot(224);  meshc(values)  # contour lines beneath the mesh
show()

#hardcopy('mesh1c.eps', color=True)
#hardcopy('mesh1c.png', color=True)

raw_input('Press Return key to quit: ')



#!/usr/bin/env python
"""
Build the scitools++ directory, consisting of scitools and several
useful associated packages such as Doconce, IPython, Epydoc, Preprocess, etc.

NumPy is required and not a part of scitools++.

Method: plain copy of already installed packages and scripts 
(lib and bin files).
"""

MODULES = (
    'doconce',
    'scitools',
    'Gnuplot',
    'IPython',
    'Scientific',
    'epydoc',
    'preprocess',
    'matplotlib',
    )

SCRIPTS = (
    'ipython',
    'epydoc',
    'epydocgui',
    'preprocess',
    'doconce_insertdocstr',
    'doconce2format',
    'diff.pl',
    'diff.py',
    'gnuplot.bat',
    '_gnuplot.py',
    'locate_pdb.py',
    'pdb',
    'timer.py',
    )


import os, glob, sys, shutil
join = os.path.join

# directories to be used:
#
# build
#    previous-build-of-scitools++
#    scitools++
#       lib
#          doconce
#          scitools
#          IPython
#          epydoc
#          preprocess
#          ...
#       bin
#
# put scitools++/lib in PYTHONPATH
# put scitools++/bin in PATH
#
# scitools contains some py4cs modules and some new stuff

thisdir = os.getcwd()
olddir = join('build', 'previous-build-of-scitools++')
newdir = join('build', 'scitools++')
libdir = join(newdir, 'lib')
bindir = join(newdir, 'bin')
scitools_dir = join(libdir, 'scitools')
py_package_src = join(os.environ['SYSDIR'],'src','python','tools')
st_src = join(os.pardir, 'lib', 'scitools')  # source code for scitools package

# start with a fresh scitools directory and move last scitools++ to
# a "copy" directory:
if not os.path.isdir('build'):
    os.mkdir('build')
if os.path.isdir(olddir):
    shutil.rmtree(olddir)
if os.path.isdir(newdir):
    os.rename(newdir, olddir)
os.makedirs(newdir)
os.makedirs(libdir)
os.makedirs(bindir)
os.makedirs(scitools_dir)

from scitools.misc import system
        
def clean(root, filetypes=['*.pyc', '*~', '*.pyo', '*.p.py', '_update.py']):
    for type in filetypes:
        os.system("find %s -name '%s' -exec rm -rf {} \;" % (root, type))
        

def copy_installed_modules():
    """
    Copy installed modules to scitools++.
    An alternative is to copy the source of these modules and
    run multiple setup.py in scitools++ to install each dir in scitools++.
    The set-up now is that scitools++ should just be in PYTHONPATH and
    then everything is correctly installed.
    """
    print '********* copy installed modules to scitools++ **************'
    path = join(sys.prefix, 'lib', 'python' + sys.version[:3], 'site-packages')
    files = MODULES  # see top of this file
    print files
    
    files = [join(path, file) for file in files]
    pmw = join(py_package_src, 'Pmw_1_3', 'Pmw')
    files.append(pmw)
    # copy files to libdir:
    # (shutil.copytree does not work properly for this type of copy)
    cmd = 'cp -r ' + ' '.join(files) + ' ' + libdir
    print cmd
    system(cmd)

def copy_installed_scripts():
    """
    Copy installed executable scripts to the scitools++/bin directory.
    """
    print '********* copy installed executable scripts **************'
    scripts = SCRIPTS  # see top of this file
    print scripts
    for script in scripts:
        system("cp `which %s` %s" % (script, bindir))

    # fix headers (replace user-specific install path):
    for f in scripts:
        system(r'subst.py "#!.+" "#!/usr/bin/env python" %s' % \
                  join(bindir, f))
    for f in glob.glob(join(bindir, '*.old~')):
        os.remove(f)


def main():
    copy_installed_modules()
    copy_installed_scripts()
    shutil.copy('scitools++_setup.py', join(newdir, 'setup.py'))
    clean(newdir, ['*.pyc', '*~', '*.pyo', '*.p.py', '_update.py', '.svn'])
    print '\nThe scitools++ umbrella was successfully made'
    print 'Go to the build/ directory and tarpack scitools++'

if __name__ == '__main__':
    main()

    



#!/usr/bin/env python
"""
Efficient database for NumPy objects.
"""

import sys, os, pickle, re
from scitools.numpytools import *

class NumPyDB:
    def __init__(self, database_name, mode='store'):
        self.filename = database_name
        self.dn = self.filename + '.dat' # NumPy array data
        self.pn = self.filename + '.map' # positions & identifiers
        if mode == 'store':
            # bring files into existence:
            fd = open(self.dn, 'w');  fd.close()
            fm = open(self.pn, 'w');  fm.close()
        elif mode == 'load':
            # check if files are there:
            if not os.path.isfile(self.dn) or \
               not os.path.isfile(self.pn):
                raise IOError("Could not find the files %s and %s" %\
                              (self.dn, self.pn))
            # load mapfile into list of tuples:
            fm = open(self.pn, 'r')
            lines = fm.readlines()
            self.positions = []
            for line in lines:
                # first column contains file positions in the
                # file .dat for direct access, the rest of the
                # line is an identifier
                c = line.split()
                # append tuple (position, identifier):
                self.positions.append((int(c[0]),
                                       ' '.join(c[1:]).strip()))
            fm.close()

    def locate(self, identifier, bestapprox=None): # base class
        """
        Find position in files where data corresponding
        to identifier are stored.
        bestapprox is a user-defined function for computing
        the distance between two identifiers.
        """
        identifier = identifier.strip()
        # first search for an exact identifier match:
        selected_pos = -1
        selected_id = None
        for pos, id in self.positions:
            if id == identifier:
                selected_pos = pos;  selected_id = id; break
        if selected_pos == -1: # 'identifier' not found?
            if bestapprox is not None:
                # find the best approximation to 'identifier':
                min_dist = \
                    bestapprox(self.positions[0][1], identifier)
                for pos, id in self.positions:
                    d = bestapprox(id, identifier)
                    if d <= min_dist:
                        selected_pos = pos;  selected_id = id
                        min_dist = d
        return selected_pos, selected_id

    def dump(self, a, identifier):  # empty base class func.
        """Dump NumPy array a with identifier."""
        raise NameError("dump is not implemented; must be impl. in subclass")

    def load(self, identifier, bestapprox=None):
        """Load NumPy array with identifier or find best approx."""
        raise NameError("load is not implemented; must be impl. in subclass")


class NumPyDB_text(NumPyDB):
    """Use plain ASCII string representation."""

    def __init__(self, database_name, mode='store'):
        NumPyDB.__init__(self, database_name, mode)

    # simple dump:
    def dump(self, a, identifier):
        """Dump NumPy array a with identifier."""
        fd = open(self.dn, 'a');  fm = open(self.pn, 'a')
        fm.write("%d\t\t %s\n" % (fd.tell(), identifier))
        fd.write(repr(a))
        fd.close();  fm.close()

    # more efficient dump (due to Mario Pernici <Mario.Pernici@mi.infn.it>)
    def dump(self, a, identifier):
        """Dump NumPy array a with identifier."""
        fd = open(self.dn, 'a');  fm = open(self.pn, 'a')
        fm.write("%d\t\t %s\n" % (fd.tell(), identifier))
        fmt = 'array([' + '%s,'*(a.size-1) + '%s])\n'
        fd.write(fmt % tuple(ravel(a)))
        fd.close();  fm.close()


    def load(self, identifier, bestapprox=None):
        """
        Load NumPy array with a given identifier. In case the
        identifier is not found, bestapprox != None means that
        an approximation is sought. The bestapprox argument is
        then taken as a function that can be used for computing
        the distance between two identifiers id1 and id2.
        """
        pos, id = self.locate(identifier, bestapprox)
        if pos < 0: return [None, "not found"]
        fd = open(self.dn, 'r')
        fd.seek(pos)
        # load the correct number of bytes; look at the next pos
        # value in self.positions (impossible if a dictionary is
        # used for self.positions - we need the order of the items!)
        for j in range(len(self.positions)):
            p = self.positions[j][0]
            if p == pos:
                try:
                    s = fd.read(self.positions[j+1][0] - p)
                except IndexError:
                    # last self.positions entry reached,
                    # just read the rest of the file:
                    s = fd.read()
                break
        a = eval(s)
        fd.close()
        return [a, id]


class NumPyDB_pickle (NumPyDB):
    """Use basic Pickle class."""

    def __init__(self, database_name, mode='store'):
        NumPyDB.__init__(self,database_name, mode)

    def dump(self, a, identifier):
        """Dump NumPy array a with identifier."""
        fd = open(self.dn, 'a');  fm = open(self.pn, 'a')
        fm.write("%d\t\t %s\n" % (fd.tell(), identifier))
        pickle.dump(a, fd, 1)  # 1: binary storage
        fd.close();  fm.close()

    def load(self, identifier, bestapprox=None):
        """
        Load NumPy array with a given identifier. In case the
        identifier is not found, bestapprox != None means that
        an approximation is sought. The bestapprox argument is
        then taken as a function that can be used for computing
        the distance between two identifiers id1 and id2.
        """
        pos, id = self.locate(identifier, bestapprox)
        if pos < 0: return None, "not found"
        fd = open(self.dn, 'r')
        fd.seek(pos)
        a = pickle.load(fd)
        fd.close()
        return a, id

import cPickle

class NumPyDB_cPickle (NumPyDB):
    """Use basic cPickle class."""

    def __init__(self, database_name, mode='store'):
        NumPyDB.__init__(self,database_name, mode)

    def dump(self, a, identifier):
        """Dump NumPy array a with identifier."""
        # fd: datafile, fm: mapfile
        fd = open(self.dn, 'a');  fm = open(self.pn, 'a')
        # fd.tell(): return current position in datafile
        fm.write("%d\t\t %s\n" % (fd.tell(), identifier))
        cPickle.dump(a, fd, 1)  # 1: binary storage
        fd.close();  fm.close()

    def load(self, identifier, bestapprox=None):
        """
        Load NumPy array with a given identifier. In case the
        identifier is not found, bestapprox != None means that
        an approximation is sought. The bestapprox argument is
        then taken as a function that can be used for computing
        the distance between two identifiers id1 and id2.
        """
        pos, id = self.locate(identifier, bestapprox)
        if pos < 0: return [None, "not found"]
        fd = open(self.dn, 'r')
        fd.seek(pos)
        a = cPickle.load(fd)
        fd.close()
        return [a, id]


import shelve

class NumPyDB_shelve:
    """Implement the database via shelving."""

    def __init__(self, database_name, mode='store'):
        self.filename = database_name # no suffix, only one file
        if mode == 'load':
            # since the keys() function in a shelf object
            # is slow, we store the keys:
            fd = shelve.open(self.filename)
            self.keys = fd.keys()
            fd.close()

    def dump(self, a, identifier):
        """Dump NumPy array a with identifier."""
        identifier = identifier.strip()
        fd = shelve.open(self.filename)
        fd[identifier] = a
        fd.close()

    def locate(self, identifier, bestapprox=None):
        """Return identifier key in shelf."""
        selected_id = None
        identifier = identifier.strip()
        if identifier in self.keys:
            selected_id = identifier
        else:
            if bestapprox:
                # find the best approximation to 'identifier':
                min_dist = bestapprox(self.keys[0], identifier)
                for id in self.keys:
                    d = bestapprox(id, identifier)
                    if d <= min_dist:
                        selected_id = id
                        min_dist = d
        return selected_id

    def load(self, identifier, bestapprox=None):
        """
        Load NumPy array with a given identifier. In case the
        identifier is not found, bestapprox != None means that
        an approximation is sought. The bestapprox argument is
        then taken as a function that can be used for computing
        the distance between two identifiers id1 and id2.
        """
        id = self.locate(identifier, bestapprox)
        if not id: return None, "not found"
        fd = shelve.open(self.filename)
        a = fd[id]
        fd.close()
        return a, id

# np.load/dump
# joblib.load/dump

def float_dist(id1, id2):
    """
    Compute distance between two identities for NumPyDB.
    Assumption: id1 and id2 are real numbers (but always sent
    as strings).
    This function is typically used when time values are
    used as identifiers.
    """
    return abs(float(id1) - float(id2))


def test_dist(id1, id2):
    """
    Return distance between identifiers id1 and id2.
    The identifiers are of the form 'time=some number'.
    """
    # extract the numbers using regex:
    #t1 = re.search(r"time=(.*)", id1).group(1)
    #t2 = re.search(r"time=(.*)", id2).group(1)
    t1 = id1[5:];  t2 = id2[5:]
    d = abs(float(t1) - float(t2))
    return d

def main(n, length, method, name):
    out = "dumping/loading %d %d-arrays data with the %s method took" \
          % (n,length,method)
    if method == "pickle":
        dataout = NumPyDB_pickle(name, 'store')
    elif method == "cPickle":
        dataout = NumPyDB_cPickle(name, 'store')
    elif method == "shelve":
        dataout = NumPyDB_shelve(name, 'store')
    elif method == "text":
        dataout = NumPyDB_text(name, 'store')
    else:
        raise ValueError("illegal method name='%s'" % method)

    import time
    t0 = time.clock()
    for i in range(n):
        u = arange(i, length/2+i, 0.4999999)
        # (generate numbers with many digits so repr(u) has
        # a representative size (not just integers, for instance))

        dataout.dump(u, 'time=%e' % float(i))

    if method == "pickle":
        datain = NumPyDB_pickle(name, 'load')
    elif method == "cPickle":
        datain = NumPyDB_cPickle(name, 'load')
    elif method == "shelve":
        datain = NumPyDB_shelve(name, 'load')
    elif method == "text":
        datain = NumPyDB_text(name, 'load')
    else:
        raise ValueError("illegal method name='%s'" % method)

    w = datain.load('time=4')
    print "identifier='time=4':", w
    # not found, no exact match for 't=4', should have
    # 'time=4.000000e+00'
    w = datain.load('time=4.000000e+00')
    print "identifier='time=4.000000e+00': found"
    if len(w[0]) < 20: print w[0]

    w = datain.load('time=5', bestapprox=test_dist)
    print "identifier='time=5' and bestapprox=test_dest found"
    if len(w[0]) < 20: print w[0]
    t1 = time.clock()
    print "%s %.2f s" % (out, t1-t0)
    if os.path.isfile(name+'.dat'):
        filesize = os.path.getsize(name+'.dat')/1000000.0
    elif os.path.isfile(name):  # shelve technique leads to no extension
        filesize = os.path.getsize(name)/1000000.0
    print "filesize=%.2fMb\n\n" % filesize
    for filename in (name+'.dat', name+'.map', name):
        if os.path.isfile(filename):
            os.remove(filename)

if __name__ == '__main__':
    try:     n = int(sys.argv[1])
    except:  n = 12
    try:     length = int(sys.argv[2])
    except:  length = 10
    try:     methods = [sys.argv[3]]
    except:  methods = ['pickle','cPickle','shelve','text']
    print 'NumPy array type:', basic_NumPy
    for method in methods:
        main(n, length, method, "tmpdata_" + method)



import unittest
import time

from scitools.numpyutils import *
from scitools.easyviz import *

import random

hardcopy_counter = 0
psplot = False
prompt = ''
if backend == 'gnuplot':
    pause = 0.2
else:
    pause = 0.0
clear_figure = False
new_figure = True
screenplot = True
setp(show=screenplot)

def next(clear_figure=False, prompt='', pause=0,
         save_hardcopy=False, new_figure=False):
    if save_hardcopy:
        global hardcopy_counter
        hardcopy('tmp_easyviz_plot%03d.eps' % hardcopy_counter, color=True)
        hardcopy_counter += 1
    if prompt:
        raw_input(prompt)
    if pause:
        time.sleep(pause)
    if clear_figure:
        clf()
    if new_figure:
        figure()
        
n = lambda: next(clear_figure, prompt, pause, psplot, new_figure)


class EasyvizTestCase(unittest.TestCase):
    def get_line_data(self, n=41):
        x = linspace(0, 15, n)
        y = sin(x)*x
        v = sin(x)*sqrt(x)
        w = sin(x)*x**0.33333333
        return x, y, v, w

    def get_3D_line_data(self, n=41):
        t = linspace(0,2*pi,40)
        x = sin(t)
        y = cos(t)
        return x, y, t

    def get_format_string_data(self):
        colors = Line._colors
        try: colors.remove('w')  # remove white
        except: pass
        markers = Line._markers
        linestyles = Line._linestyles
        format = []
        for marker in markers:
            format.append(''.join([random.choice(colors), marker,
                                   random.choice(linestyles)]))
        return format
    
    def get_2D_data(self, n=21, sparse=True, indexing='ij'):
        x = y = linspace(-3,3,n)
        xv, yv = meshgrid(x, y, sparse=sparse, indexing=indexing)
        values = peaks(xv, yv)
        return x, y, xv, yv, values

    def get_3D_data(self, sparse=True, indexing='ij'):
        x = y = z = linspace(-3,3,13)
        xv, yv, zv = meshgrid(x, y, z, sparse=sparse, indexing=indexing)
        values = xv**exp(-xv**2-yv**2-zv**2)
        return x, y, z, xv, yv, zv, values

    def get_2D_vector_data(self, n=13, sparse=True, indexing='ij'):
        x = y = linspace(-2,2,n)
        xv, yv = meshgrid(x, y, sparse=sparse, indexing=indexing)
        values = xv*exp(-xv**2-yv**2)
        uv, vv = gradient(values)
        return x, y, xv, yv, values, uv, vv

    def get_3D_vector_data(self, sparse=True, indexing='ij'):
        x = y = z = linspace(-3,3,13)
        xv, yv, zv = meshgrid(x, y, z, sparse=sparse, indexing=indexing)
        values = xv**exp(-xv**2-yv**2-zv**2)
        
        return x, y, z, xv, yv, zv, values
        



"""Test functions for contour3 command."""

from easyviztest import *


class test_contour3_basic(EasyvizTestCase):
    def check_contour3_default(self):
        x,y,xv,yv,values = self.get_2D_data()
        contour3(xv,yv,values)
        title("contour3(xv,yv,values)")
        n()

    def check_contour3_no_grid_components(self):
        x,y,xv,yv,values = self.get_2D_data()
        contour3(values)
        title("contour3(values)")
        n()

    def check_contour3_1D_grid_components(self):
        x,y,xv,yv,values = self.get_2D_data()
        contour3(x,y,values)
        title("contour3(xv,yv,values)")
        n()

    def check_contour3_not_sparse_grid(self):
        x,y,xv,yv,values = self.get_2D_data(sparse=False)
        contour3(xv,yv,values)
        title("contour3(xv,yv,values) (xv and yv not sparse)")
        n()


class test_contour3_levels(EasyvizTestCase):
    def check_contour3_5_levels(self):
        x,y,xv,yv,values = self.get_2D_data()
        contour3(xv,yv,values,5)
        title("contour3(xv,yv,values,5)")
        n()

    def check_contour3_15_levels_kwarg(self):
        x,y,xv,yv,values = self.get_2D_data()
        contour3(xv,yv,values,clevels=15)
        title("contour3(xv,yv,values,clevels=15)")
        n()

    def check_contour3_specify_levels(self):
        x,y,xv,yv,values = self.get_2D_data()
        contour3(xv,yv,values,[-3, -1, 1, 5])
        title("contour3(xv,yv,values,[-3, -1, 1, 5])")
        n()

    def check_contour3_specify_levels(self):
        x,y,xv,yv,values = self.get_2D_data()
        contour3(xv,yv,values,cvector=linspace(-10,10,15))
        title("contour3(xv,yv,values,cvector=linspace(-10,10,15))")
        n()
        

class test_contour3_indexing_xy(EasyvizTestCase):
    def check_contour3_indexing_xy_default(self):
        x,y,xv,yv,values = self.get_2D_data(indexing='xy')
        contour3(xv,yv,values,indexing='xy')
        title("contour3(xv,yv,values,indexing='xy')")
        n()
        
    def check_contour3_indexing_xy_rectangular_grid(self):
        x = linspace(-3,3,21)
        y = linspace(-2,2,13)
        xv, yv = meshgrid(x, y, indexing='xy')
        values = peaks(xv,yv)
        contour3(xv,yv,values,indexing='xy')
        title("contour3(xv,yv,values,indexing='xy') (rectangular grid)")
        n()
        
    def check_contour3_indexing_xy_rectangular_grid2(self):
        x = linspace(-5,5,21)
        y = linspace(-2,2,31)
        xv, yv = meshgrid(x, y, indexing='xy')
        values = peaks(xv,yv)
        contour3(xv,yv,values,indexing='xy')
        title("contour3(xv,yv,values,indexing='xy') (rectangular grid)")
        n()


class test_contour3_misc(EasyvizTestCase):
    def check_contour3_format_string(self):
        x,y,xv,yv,values = self.get_2D_data()
        contour3(xv,yv,values,'b:')
        title("contour3(..,'b:')")
        n()

    def check_contour3_12_levels_format_string(self):
        x,y,xv,yv,values = self.get_2D_data()
        contour3(xv,yv,values,12,'r--')
        title("contour3(..,12,'r--')")
        n()

    def check_contour3_linewidth(self):
        x,y,xv,yv,values = self.get_2D_data()
        contour3(xv,yv,values,linewidth=3)
        title("contour3(..,linewidth=3)")
        n()


class test_contour3_labels(EasyvizTestCase):
    def check_contour3_labels_on(self):
        x,y,xv,yv,values = self.get_2D_data()
        h = contour3(xv,yv,values)
        clabel(h,'on')
        title("h=contour3(..);clabel(h,'on')")
        n()

    def check_contour3_labels_off(self):
        x,y,xv,yv,values = self.get_2D_data()
        h = contour3(xv,yv,values)
        clabel(h,'on')
        clabel(h,'off')
        title("h=contour3(..);clabel(h,'on');clabel(h,'off')")
        n()

    def check_contour3_labels_on_kwarg(self):
        x,y,xv,yv,values = self.get_2D_data()
        contour3(xv,yv,values,clabels='on')
        title("contour3(..,clabels='on')")
        n()


def test_suite(level=1):
    suites = []
    if level > 0:
        suites.append(unittest.makeSuite(test_contour3_basic,'check_'))
        suites.append(unittest.makeSuite(test_contour3_levels,'check_'))
        suites.append(unittest.makeSuite(test_contour3_indexing_xy,
                                         'check_'))
        suites.append(unittest.makeSuite(test_contour3_misc,'check_'))
        suites.append(unittest.makeSuite(test_contour3_labels,'check_'))
    total_suite = unittest.TestSuite(suites)
    return total_suite

def test(level=10):
    all_tests = test_suite()
    runner = unittest.TextTestRunner()
    runner.run(all_tests)
    return runner

if __name__ == "__main__":
    test()
    raw_input("press enter to exit")



"""
Exemplify how a vector can be rotated in the plane.
Method: display one vector as a vector field, using quiver,
with a 1x1 cells in the grid and vanishing vectors except
at the first grid point (origin).
The example show how the quiver function can be used to
display single vectors in a coordinate system.
"""
from scitools.std import *

vec1 = matrix([1, 2], dtype=float).transpose()
theta = pi/3  # rotation angle
transformation = matrix([[cos(theta), -sin(theta)], [sin(theta), cos(theta)]])
print transformation
vec2 = transformation*vec1

# Use quiver to plot vector fields on a 1x1 grid with the field
# being 0 at all points except the origin.

def vec2vecfield(vec, shape):
    """
    Given a vector and the shape (M,N) of an MxN grid,
    return a 2D vector field which is (0,0) at all points
    except for the origin where the field equals vec.
    """
    u, v = zeros(shape), zeros(shape)  # vector field
    u[0,0] = vec[0]; v[0,0] = vec[1]
    return u, v

# Make 1x1 grid
axis_extent_x = 2
axis_extent_y = 2
x = linspace(0, axis_extent_x, 2)  # no of cells: 1x1
y = linspace(0, axis_extent_y, 2)  # no of cells: 1x1
xv, yv = ndgrid(x, y, sparse=False)

u1, v1 = vec2vecfield(vec1, xv.shape)
quiver(xv, yv, u1, v1, 'b')  # b for blue vector
hold('on')
u2, v2 = vec2vecfield(vec2, xv.shape)
quiver(xv, yv, u2, v2, 'r')  # r for red vector



"""Test functions for surf command."""

from easyviztest import *


class test_surf_basic(EasyvizTestCase):
    def check_surf_default(self):
        x,y,xv,yv,values = self.get_2D_data()
        surf(xv,yv,values)
        title("surf(xv,yv,values)")
        n()

    def check_surf_no_grid_components(self):
        x,y,xv,yv,values = self.get_2D_data()
        surf(values)
        title("surf(values)")
        n()
        
    def check_surf_1D_grid_components(self):
        x,y,xv,yv,values = self.get_2D_data()
        surf(x,y,values)
        title("surf(x,y,values)")
        n()
        
    def check_surf_not_sparse_grid(self):
        x,y,xv,yv,values = self.get_2D_data(sparse=False)
        surf(xv,yv,values)
        title("surf(xv,yv,values) (xv and yv not sparse)")
        n()
        
    def check_surf_indexing_xy(self):
        x,y,xv,yv,values = self.get_2D_data(indexing='xy')
        surf(xv,yv,values,indexing='xy')
        title("surf(xv,yv,values,indexing='xy')")
        n()


class test_surf_rectangular_grid(EasyvizTestCase):
    def check_surf_rectangular_grid(self):
        x = linspace(-2,2,21)
        y = linspace(-3,3,13)
        xv, yv = ndgrid(x, y)
        values = peaks(xv,yv)
        surf(xv,yv,values)
        title("surf(xv,yv,values) (rectangular grid)")
        n()

    def check_surf_rectangular_grid_no_grid_components(self):
        x = linspace(-2,2,21)
        y = linspace(-3,3,13)
        xv, yv = ndgrid(x, y)
        values = peaks(xv, yv)
        surf(values)
        title("surf(values) (rectangular grid)")
        n()
       
    def check_surf_rectangular_grid2(self):
        x = linspace(-5,5,41)
        y = linspace(-2,2,9)
        xv, yv = ndgrid(x, y)
        values = peaks(xv, yv)
        surf(xv,yv,values)
        title("surf(xv,yv,values) (rectangular grid)")
        n()
       
    def check_surf_rectangular_grid2_no_grid_components(self):
        x = linspace(-5,5,41)
        y = linspace(-2,2,9)
        xv, yv = ndgrid(x, y)
        values = peaks(xv, yv)
        surf(xv,yv,values)
        title("surf(xv,yv,values) (rectangular grid)")
        n()
        
    def check_surf_rectangular_grid_indexing_xy(self):
        x = linspace(-2,2,21)
        y = linspace(-3,3,13)
        xv, yv = meshgrid(x, y, indexing='xy')
        values = peaks(xv, yv)
        surf(xv,yv,values,indexing='xy')
        title("surf(xv,yv,values,indexing='xy') (rectangular grid)")
        n()
        
    def check_surf_rectangular_grid2_indexing_xy(self):
        x = linspace(-5,5,41)
        y = linspace(-2,2,9)
        xv, yv = meshgrid(x, y, indexing='xy')
        values = peaks(xv, yv)
        surf(xv,yv,values,indexing='xy')
        title("surf(xv,yv,values,indexing='xy') (rectangular grid)")
        n()


## class test_surf_multiple_surfaces(EasyvizTestCase):
##     def check_surf_two_surfaces(self):
##         x,y,xv,yv,values = self.get_2D_data(sparse=False)
##         setp(show=False)
##         surf(xv,yv,xv*yv)
##         hold('on')
##         surf(xv,yv,xv**2+yv**2)
##         title("surf(xv,yv,xv*yv);hold('on');surf(xv,yv,xv**2+yv**2)")
##         hold('off')
##         setp(show=screenplot)
##         if screenplot:
##             show()
##         n()


class test_surf_cdata(EasyvizTestCase):
    def check_surf_with_cdata(self):
        x,y,xv,yv,values = self.get_2D_data(sparse=False)
        cdata = xv
        surf(xv,yv,values,cdata)
        title("cdata=xv;surf(xv,yv,values,cdata)")
        n()

    def check_surf_with_cdata2(self):
        x,y,xv,yv,values = self.get_2D_data(sparse=False)
        cdata = values
        values = xv
        surf(xv,yv,values,cdata)
        title("cdata=values;values=xv;surf(xv,yv,values,cdata)")
        n()


class test_surf_shading(EasyvizTestCase):
    def check_surf_shading_faceted(self):
        x,y,xv,yv,values = self.get_2D_data()
        surf(xv,yv,values)
        shading('faceted')
        title("surf(xv,yv,values);shading('faceted')")
        n()

    def check_surf_shading_faceted_kwarg(self):
        x,y,xv,yv,values = self.get_2D_data()
        surf(xv,yv,values,shading='faceted')
        title("surf(xv,yv,values,shading='faceted')")
        n()

    def check_surf_shading_flat(self):
        x,y,xv,yv,values = self.get_2D_data()
        surf(xv,yv,values)
        shading('flat')
        title("surf(xv,yv,values);shading('flat')")
        n()

    def check_surf_shading_flat_kwarg(self):
        x,y,xv,yv,values = self.get_2D_data()
        surf(xv,yv,values,shading='flat')
        title("surf(xv,yv,values,shading='flat')")
        n()
        
    def check_surf_shading_interp(self):
        x,y,xv,yv,values = self.get_2D_data()
        surf(xv,yv,values)
        shading('interp')
        title("surf(xv,yv,values);shading('interp')")
        n()

    def check_surf_shading_interp_kwarg(self):
        x,y,xv,yv,values = self.get_2D_data()
        surf(xv,yv,values,shading='interp')
        title("surf(xv,yv,values,shading='interp')")
        n()


def test_suite(level=1):
    suites = []
    if level > 0:
        suites.append(unittest.makeSuite(test_surf_basic,'check_'))
        suites.append(unittest.makeSuite(test_surf_rectangular_grid,'check_'))
        suites.append(unittest.makeSuite(test_surf_cdata,'check_'))
        suites.append(unittest.makeSuite(test_surf_shading,'check_'))
    total_suite = unittest.TestSuite(suites)
    return total_suite

def test(level=10):
    all_tests = test_suite()
    runner = unittest.TextTestRunner()
    runner.run(all_tests)
    return runner

if __name__ == "__main__":
    test()
    raw_input("press enter to exit")



"""Test functions for quiver command."""

from easyviztest import *


class test_quiver_basic(EasyvizTestCase):
    def check_quiver_default(self):
        x,y,xv,yv,values,uv,vv = self.get_2D_vector_data()
        quiver(xv,yv,uv,vv)
        title("quiver(xv,yv,uv,vv)")
        n()

    def check_quiver_not_sparse_grid(self):
        x,y,xv,yv,values,uv,vv = self.get_2D_vector_data(sparse=False)
        quiver(xv,yv,uv,vv)
        title("quiver(xv,yv,uv,vv) (xv and yv not sparse)")
        n()

    def check_quiver_1D_grid_components(self):
        x,y,xv,yv,values,uv,vv = self.get_2D_vector_data()
        quiver(x,y,uv,vv)
        title("quiver(x,y,uv,vv)")
        n()
    
    def check_quiver_no_grid_components(self):
        x,y,xv,yv,values,uv,vv = self.get_2D_vector_data()
        quiver(uv,vv)
        title("quiver(uv,vv)")
        n()

    def check_quiver_1D_vector_components(self):
        x,y,xv,yv,values,uv,vv = self.get_2D_vector_data()
        quiver(x,y,x,y**2)
        title("quiver(x,y,x,y**2)")
        n()
    

class test_quiver_scaling(EasyvizTestCase):
    def check_quiver_no_automatic_scaling(self):
        x,y,xv,yv,values,uv,vv = self.get_2D_vector_data()
        quiver(xv,yv,uv,vv,0)
        title("quiver(xv,yv,uv,vv,0)")
        n()

    def check_quiver_half_size(self):
        x,y,xv,yv,values,uv,vv = self.get_2D_vector_data()
        quiver(xv,yv,uv,vv,0.5)
        title("quiver(xv,yv,uv,vv,0.5)")
        n()

    def check_quiver_double_size(self):
        x,y,xv,yv,values,uv,vv = self.get_2D_vector_data()
        quiver(xv,yv,uv,vv,2)
        title("quiver(xv,yv,uv,vv,2)")
        n()

    def check_quiver_quadruple_size(self):
        x,y,xv,yv,values,uv,vv = self.get_2D_vector_data()
        quiver(xv,yv,uv,vv,4)
        title("quiver(xv,yv,uv,vv,4)")
        n()


class test_quiver_indexing_xy(EasyvizTestCase):
    def check_quiver_indexing_xy_default(self):
        x,y,xv,yv,values,uv,vv = self.get_2D_vector_data(indexing='xy')
        quiver(xv,yv,uv,vv,indexing='xy')
        title("quiver(xv,yv,uv,vv,indexing='xy')")
        n()

    def check_quiver_indexing_xy_no_grid_components(self):
        x,y,xv,yv,values,uv,vv = self.get_2D_vector_data(indexing='xy')
        quiver(uv,vv,indexing='xy')
        title("quiver(uv,vv,indexing='xy')")
        n()    

    def check_quiver_indexing_xy_1D_grid_components(self):
        x,y,xv,yv,values,uv,vv = self.get_2D_vector_data(indexing='xy')
        quiver(x,y,uv,vv,indexing='xy')
        title("quiver(x,y,uv,vv,indexing='xy')")
        n()


class test_quiver_misc(EasyvizTestCase):
    def check_quiver_scaling_fmt(self):
        x,y,xv,yv,values,uv,vv = self.get_2D_vector_data()
        quiver(xv,yv,uv,vv,2,'ko')
        title("quiver(xv,yv,uv,vv,2,'ko')")
        n()

    def check_quiver_scaling_filled(self):
        x,y,xv,yv,values,uv,vv = self.get_2D_vector_data()
        quiver(xv,yv,uv,vv,2.8,'filled')
        title("quiver(xv,yv,uv,vv,2.8,'filled')")
        n()

    def check_quiver_scaling_filled_fmt(self):
        x,y,xv,yv,values,uv,vv = self.get_2D_vector_data()
        quiver(xv,yv,uv,vv,3,'filled','m')
        title("quiver(xv,yv,uv,vv,3,'filled','m')")
        n()

    def check_quiver_scaling_fmt_filled(self):
        x,y,xv,yv,values,uv,vv = self.get_2D_vector_data()
        quiver(xv,yv,uv,vv,3,'cd','filled')
        title("quiver(xv,yv,uv,vv,3,'cd','filled')")
        n()

    def check_quiver_no_grid_components_scaling_filled_fmt(self):
        x,y,xv,yv,values,uv,vv = self.get_2D_vector_data()
        quiver(uv,vv,2,'filled','g3')
        title("quiver(uv,vv,2,'filled','g3')")
        n()

    def check_quiver_filled_fmt(self):
        x,y,xv,yv,values,uv,vv = self.get_2D_vector_data()
        quiver(xv,yv,uv,vv,'filled','ko')
        title("quiver(xv,yv,uv,vv,'filled','ko')")
        n()

    def check_quiver_fmt_filled(self):
        x,y,xv,yv,values,uv,vv = self.get_2D_vector_data()
        quiver(xv,yv,uv,vv,'y--','filled')
        title("quiver(xv,yv,uv,vv,'y--','filled')")
        n()


def test_suite(level=1):
    suites = []
    if level > 0:
        suites.append(unittest.makeSuite(test_quiver_basic,'check_'))
        suites.append(unittest.makeSuite(test_quiver_scaling,'check_'))
        suites.append(unittest.makeSuite(test_quiver_indexing_xy,'check_'))
        suites.append(unittest.makeSuite(test_quiver_misc,'check_'))
    total_suite = unittest.TestSuite(suites)
    return total_suite

def test(level=10):
    all_tests = test_suite()
    runner = unittest.TextTestRunner()
    runner.run(all_tests)
    return runner

if __name__ == "__main__":
    test()
    raw_input("press enter to exit")



"""Make plot of a curve in pure text format (ASCII)."""
# this is the same example as in the scitools.aplotter doc string

from scitools.aplotter import plot
from numpy import linspace, exp, cos, pi
x = linspace(-2, 2, 81)
y = exp(-0.5*x**2)*cos(pi*x)
plot(x, y)

plot(x, y, draw_axes=False)

# plot symbols (the dot argument) at data points:
plot(x, y, plot_slope=False)
 
# drop axis labels:
plot(x, y, plot_labels=False)

plot(x, y, dot='o', plot_slope=False)

# store plot in a string:
p = plot(x, y, output=str)
print p



 




from scitools.std import *
# Clean up tmp_*.eps files:
import glob, os
for name in glob.glob('tmp_*.png'):
    os.remove(name)
# Clean up movie files
for name in glob.glob('tmpmovie*.*'):
    os.remove(name)

def f(x, t):
    return exp(-(x-3*t)**2)*sin(3*pi*(x-t))

xmax = 6
x = linspace(-xmax, xmax, 1001)


t_values = linspace(-1, 1, 61)
counter = 1
for t in t_values:
    y = f(x, t)
    plot(x, y, hardcopy='tmp_%04d.png' % counter)
    counter += 1

# Make movie file the simplest possible way
movie('tmp_*.png')
import glob, os
print 'generated the file', glob.glob('movie.*')[0]
#os.remove(glob.glob('movie.*')[0])

# Make animated GIF movie in the file tmpmovie.gif
movie('tmp_*.png', encoder='convert', fps=2,
      output_file='tmpmovie.gif')

# Show movie (os.system runs an operating system command)
os.system('animate tmpmovie.gif &')

# Other formats
movie('tmp_*.png', encoder='html', fps=3,
      output_file='tmpmovie.html')  # play in HTML file

movie('tmp_*.png', encoder='ppmtompeg', fps=24,
      output_file='tmpmovie1.mpeg') # requires netpbm package

movie('tmp_*.png', encoder='ffmpeg', fps=4,
      output_file='tmpmovie1b.mpeg') # requires ffmpeg package

movie('tmp_*.png', encoder='ffmpeg', fps=4,
      output_file='tmpmovie1.avi') # requires ffmpeg package

movie('tmp_*.png', encoder='ffmpeg',
      output_file='tmpmovie1c.mpeg', vodec='mpeg2video')



#!/usr/bin/env python

# Example taken from:
# http://www.mathworks.com/access/helpdesk/help/techdoc/visualize/f5-6736.html

from scitools.easyviz import *
from scipy import io

wind = io.loadmat('wind_matlab_v6.mat')
x = wind['x']
y = wind['y']
z = wind['z']
u = wind['u']
v = wind['v']
w = wind['w']

xmin = arrmin(x)
xmax = arrmax(x)
ymin = arrmin(y)
alt = 7.356 # z-value for slice and streamtube plane
wind_speed = sqrt(u**2 + v**2 + w**2)

setp(interactive=False)

# Draw Slice Planes:
hslice = slice_(x,y,z,wind_speed,xmax,ymin,alt)
hold('on')
#set(hslice,'FaceColor','interp','EdgeColor','none')
ax = gca()
ax.setp(shading='interp')
colormap(hsv(16))

# Add Contour Lines to Slice Planes:
#color_lim = caxis()
# BUG: caxis() returns [None,None]
color_lim = [0.2681,78.9072] 
cont_intervals = linspace(color_lim[0],color_lim[1],17)
hcont = contourslice(x,y,z,wind_speed,xmax,ymin,alt,cont_intervals.tolist())
# BUG: In Volume._parseargs_slice_ in common.py line 932:
#                 elif isinstance(tmparg, (list,tuple)):
# should also accept NumPy arrays!

#set(hcont,'EdgeColor',[.4 .4 .4],'LineWidth',1)
hcont.setp(linewidth=2)

# Create the Stream Tubes:
sx,sy,sz = ndgrid([xmin]*11,seq(20,50,3),[alt]*11)
daspect([1,1,1]) # set DAR before calling streamtube
htubes = streamtube(x,y,z,u,v,w,sx,sy,sz)#,[1.25, 5])
#set(htubes,'EdgeColor','none','FaceColor','r','AmbientStrength',.5)

# Define the view:
view(-100,30)
#axis(volumebounds(x,y,z,wind_speed))
#set(gca,'Projection','perspective')
ax.setp(projection='perspective')
#camlight left
grid('off')

show()

raw_input('Press Return key to quit: ')

#hardcopy('tmp_streamtube2.eps')
#hardcopy('tmp_streamtube2_lq.eps', vector_file=False)
#hardcopy('tmp_streamtube2.png')






from scitools.std import *   # for curve plotting

def f(t):
    return t**2*exp(-t**2)

t = linspace(0, 3, 51)    # 51 points between 0 and 3
y = zeros(len(t), 'd')    # 51 doubles ('d')
for i in xrange(len(t)):
    y[i] = f(t[i])

plot(t, y)

raw_input('Press Return key to quit: ')



allow_plot = True
from scitools.std import *

def test_PiecewiseConstant():
    data = [(0, 1.5), (3, 8), (4, 1)]
    f = PiecewiseConstant(domain=[0, 7],
                          data=data,
                          eps=0)
    b = [x for x, value in data]  # boundaries
    b.append(7)
    b = array(b)
    v = array([value for x, value in data])
    xm = array([0.5*(b[i-1] + b[i]) for i in range(1, len(b))]) # midpoints

    # Scalar vs array x in f(x)
    x = xm[0]
    assert allclose(f(xm), f(array(xm)))
    # Evaluations in the interior of domains
    assert allclose(f(xm), f._values)
    # Check boundary values: f(x[:-1]) = v, f(x[-1]) = v[-1]
    x = b
    y = zeros(len(v)+1)
    y[:-1] = v
    y[-1] = v[-1]
    assert allclose(f(x), y)

    if allow_plot:
        x = linspace(f.L, f.R, 31)    # no hit of boundaries
        y = f(x)
        x1, y1 = f.plot()
        n = 2
        x2 = linspace(f.L, f.R, n*7+1)  # hit of boundaries
        y2 = f(x2)
        plot(x, y, 'ro',
             x1, y1, 'b-',
             x2, y2, 'ko',
             legend=('f(x) w/no boundary pts in x', 'plot',
                     'f(x) w/boundary pts'),
             axis=[x[0], x[-1], 0, 10])

    f = PiecewiseConstant(domain=[0, 7],
                          data=data,
                          eps=0.5)

    # Scalar vs array x in f(x)
    x = xm[0]
    assert allclose(f(xm), f(array(xm)))
    # Evaluations in the interior of domains
    assert allclose(f(xm), f._values, atol=1E-6)
    # Check boundary values: mean values
    x = b
    y = zeros(len(v)+1)
    y[0] = v[0]
    y[1:-1] = 0.5*(v[:-1] + v[1:])
    y[-1] = v[-1]
    assert allclose(f(x), y)

    if allow_plot:
        figure()
        x = linspace(f.L, f.R, 31)    # no hit of boundaries
        y = f(x)
        x1, y1 = f.plot()
        n = 2
        x2 = linspace(f.L, f.R, n*7+1)  # hit of boundaries
        y2 = f(x2)
        plot(x, y, 'ro',
             x1, y1, 'b-',
             x2, y2, 'ko',
             legend=('f(x) w/no boundary pts in x', 'plot',
                     'f(x) w/boundary pts'),
             axis=[x[0], x[-1], 0, 10])

test_PiecewiseConstant()




from scitools.std import *

def f1(t):
    return t**2*exp(-t**2)

def f2(t):
    return t**2*f1(t)

t = linspace(0, 3, 51)
y1 = f1(t)
y2 = f2(t)

# Pick out each 4 points and add random noise
t3 = t[::4]      # slice, stride 4
random.seed(11)  # fix random sequence
noise = random.normal(loc=0, scale=0.02, size=len(t3))
y3 = y2[::4] + noise

plot(t, y1, 'r-')
hold('on')
plot(t, y2, 'ks-')
plot(t3, y3, 'bo')

legend('t^2*exp(-t^2)', 't^4*exp(-t^2)', 'data')
title('Simple Plot Demo of t^2*exp(-t^2) and t^4*exp(-t^2) functions')
axis([0, 3, -0.05, 0.6])
xlabel('t')
ylabel('y')
savefig('tmp3.eps') 
savefig('tmp3.png')
show()

raw_input('Press Return key to quit: ')



"""Test functions for colorbar command."""

from easyviztest import *


class test_colorbar_basic(EasyvizTestCase):
    def check_colorbar_default(self):
        x,y,xv,yv,values = self.get_2D_data()
        pcolor(xv,yv,values)
        colorbar()
        title("pcolor(..);colorbar()")
        n()

    def check_colorbar_off(self):
        x,y,xv,yv,values = self.get_2D_data()
        pcolor(xv,yv,values)
        colorbar()
        colorbar('off')
        title("pcolor(..);colorbar();colorbar('off')")
        n()

    def check_colorbar_default_kwarg(self):
        x,y,xv,yv,values = self.get_2D_data()
        pcolor(xv,yv,values,colorbar='on')
        title("pcolor(..,colorbar='on')")
        n()

    def check_colorbar_cbtitle(self):
        x,y,xv,yv,values = self.get_2D_data()
        pcolor(xv,yv,values,colorbar='on',cbtitle='colorbar title')
        title("pcolor(..,colorbar='on',cbtitle='colorbar title')")
        n()

    def check_colorbar_axes_specified(self):
        x,y,xv,yv,values = self.get_2D_data()
        pcolor(xv,yv,values)
        colorbar(gca())
        title("pcolor(..);colorbar(gca())")
        n()

    def check_colorbar_off_axes_specified(self):
        x,y,xv,yv,values = self.get_2D_data()
        pcolor(xv,yv,values)
        colorbar(gca())
        colorbar(gca(),'off')
        title("pcolor(..);colorbar(gca());colorbar(gca(),'off')")
        n()


class test_colorbar_location(EasyvizTestCase):
    def check_colorbar_location(self):
        x,y,xv,yv,values = self.get_2D_data()
        cbar = getp(gca(), 'colorbar')
        for location in cbar._locations:
            pcolor(xv,yv,values)
            colorbar(location)
            title("pcolor(..);colorbar('%s')" % location)
            n()
            
    def check_colorbar_location_kwarg(self):
        x,y,xv,yv,values = self.get_2D_data()
        cbar = getp(gca(), 'colorbar')
        for location in cbar._locations:
            pcolor(xv,yv,values,colorbar='on',cblocation='%s' % location)
            title("pcolor(..,colorbar='on',cblocation='%s')" % location)
            n()


def test_suite(level=1):
    suites = []
    if level > 0:
        suites.append(unittest.makeSuite(test_colorbar_basic,'check_'))
        suites.append(unittest.makeSuite(test_colorbar_location,'check_'))
    total_suite = unittest.TestSuite(suites)
    return total_suite

def test(level=10):
    all_tests = test_suite()
    runner = unittest.TextTestRunner()
    runner.run(all_tests)
    return runner

if __name__ == "__main__":
    test()
    raw_input("press enter to exit")
